#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if env.get('CM_PYTHON_CONDA', '') == 'yes' and env.get(
            'CM_CONDA_BIN_PATH', '') != '':
        env['CM_PYTHON_BIN_WITH_PATH'] = os.path.join(
            env['CM_CONDA_BIN_PATH'], "python")

    recursion_spaces = i['recursion_spaces']

    # we need to understand whether this script is called first and CM_PYTHON_BIN_WITH_PATH is empty
    #   then we should search for related artifacts (python in our case)
    # or this script is called after install-python* and CM_PYTHON_BIN_WITH_PATH is set there
    # then we do not search for an artifact (python) but pick it up from the
    # installation

    if 'CM_PYTHON_BIN_WITH_PATH' not in env:
        # file_name = 'python.exe' if os_info['platform'] == 'windows' else 'python[0-9|\.]*$'
        file_name = 'python.exe' if os_info['platform'] == 'windows' else 'python3'
        extra_paths = {"include": "+C_INCLUDE_PATH", "lib": "+LD_LIBRARY_PATH"}

        r = i['automation'].find_artifact({'file_name': file_name,
                                           'default_path_env_key': 'PATH',
                                           'env': env,
                                           'os_info': os_info,
                                           # this key defines env key with
                                           # paths where to find an artifact
                                           'detect_version': True,
                                           # the next key is used in run.sh to
                                           # detect python version
                                           'env_path_key': 'CM_PYTHON_BIN_WITH_PATH',
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': i['recursion_spaces'],
                                           'extra_paths': extra_paths
                                           })
        if r['return'] > 0:
            if r['return'] == 16 and os_info['platform'] != 'windows':
                # If artifact is not found and we are not on windows
                # we should try to install python from src
                # in prehook_deps
                env['CM_REQUIRE_INSTALL'] = "yes"

                return {'return': 0}
            else:
                return r

    return {'return': 0}


def detect_version(i):
    r = i['automation'].parse_version({'match_text': r'Python\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_PYTHON_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']

    print(i['recursion_spaces'] + '      Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    env = i['env']
    os_info = i['os_info']

    r = detect_version(i)
    if r['return'] > 0:
        return r

    version = r['version']

    found_file_path = env['CM_PYTHON_BIN_WITH_PATH']

    found_path = os.path.dirname(found_file_path)

    env['CM_PYTHON_BIN'] = os.path.basename(found_file_path)
    env['CM_PYTHON_BIN_PATH'] = os.path.dirname(found_file_path)

    # Save tags that can be used to specialize further dependencies (such as
    # python packages)
    tags = 'version-' + version

    add_extra_cache_tags = []

    extra_tags = env.get('CM_EXTRA_CACHE_TAGS', '')
    if extra_tags != '':
        tags += ',' + extra_tags

    # Check if called from virtual env installer
    from_virtual = True if 'virtual' in extra_tags.split(',') else False

    if not from_virtual:
        tags += ',non-virtual'

    env['CM_PYTHON_CACHE_TAGS'] = tags

    add_extra_cache_tags = tags.split(',')

    # Check if need to add path, include and lib to env
    # (if not in default paths)
    default_path_list = i['automation'].get_default_path_list(i)
    found_path_root = os.path.dirname(found_path)

    if from_virtual:
        # Clean PATH (it will be in activate script)
        # but keep LD_LIBRARY_PATH and C_INCLUDE_PATH from the native python
        for k in ['+PATH']:
            if k in env:
                del (env[k])

    elif os_info['platform'] == 'windows':
        extra_path = os.path.join(found_path, 'Scripts')

        if extra_path not in default_path_list and extra_path + \
                os.sep not in default_path_list:
            paths = env.get('+PATH', [])
            if extra_path not in paths:
                paths.append(extra_path)
                env['+PATH'] = paths

    version_split = version.split(".")
    python_major_version = version_split[0]
    python_minor_version = version_split[1]
    if len(version_split) > 2:
        python_patch_version = version_split[2]

    env['CM_PYTHON_MAJOR_VERSION'] = python_major_version
    env['CM_PYTHON_MINOR_VERSION'] = python_minor_version
    env['CM_PYTHON_PATCH_VERSION'] = python_patch_version

    return {'return': 0, 'version': version,
            'add_extra_cache_tags': add_extra_cache_tags}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    state = i['state']

    # If windows, download here otherwise use run.sh
    if os_info['platform'] == 'windows':

        script_prefix = state.get('script_prefix', [])

        s = '@echo off'
        if s not in script_prefix:
            script_prefix.insert(0, s)

        state['script_prefix'] = script_prefix

    # Test to skip next dependency
    # env = i['env']
    # env['CM_SKIP_SYS_UTILS'] = 'YES'

    return {'return': 0}



def check_return(r):
    if 'return' not in r:
        raise Exception(
            'CM access function should always return key \'return\'!')
    if 'error' in r:
        raise Exception(r['error'])


def check_list(r, string, found=True):
    check_return(r)
    if 'list' not in r:
        raise Exception('CM search should return a list!')
    if len(r['list']) < 1 and found:
        raise Exception('CM search returned an empty list for ' + string)
    if len(r['list']) > 0 and not found:
        raise Exception('CM search returned at lease one entry for ' + string)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

#    if os_info['platform'] == 'windows':
# return {'return':1, 'error': 'Windows is not supported in this script
# yet'}

    env = i['env']
    meta = i['meta']

    script_path = i['run_script_input']['path']

    if env.get('CM_GIT_CHECKOUT', '') == '' and env.get(
            'CM_GIT_URL', '') == '' and env.get('CM_VERSION', '') == '':
        # if custom checkout and url parameters are not set and CM_VERSION is
        # not specified
        env['CM_VERSION'] = "master"
        env["CM_GIT_CHECKOUT"] = "master"
        env["CM_GIT_URL"] = "https://github.com/mlcommons/inference"
    elif env.get('CM_GIT_CHECKOUT', '') != '' and env.get('CM_TMP_GIT_CHECKOUT', '') != '' and env.get('CM_GIT_CHECKOUT', '') != env.get('CM_TMP_GIT_CHECKOUT', ''):
        # if checkout branch is assigned inside version and custom branch is
        # also specified
        return {
            "return": 1, "error": "Conflicting branches between version assigned and user specified."}
    elif env.get('CM_GIT_URL', '') != '' and env.get('CM_TMP_GIT_URL', '') != '' and env.get('CM_GIT_URL', '') != env.get('CM_TMP_GIT_URL', ''):
        # if GIT URL is assigned inside version and custom branch is also
        # specified
        return {
            "return": 1, "error": "Conflicting URL's between version assigned and user specified."}

    if env.get('CM_VERSION', '') == '':
        env['CM_VERSION'] = "custom"

    # check whether branch and url is specified,
    # if not try to assign the values specified in version parameters,
    # if version parameters does not have the value to a parameter, set the
    # default one
    if env.get('CM_GIT_CHECKOUT', '') == '' and env.get(
            'CM_GIT_CHECKOUT_TAG', '') == '':
        if env.get('CM_TMP_GIT_CHECKOUT', '') != '':
            env["CM_GIT_CHECKOUT"] = env["CM_TMP_GIT_CHECKOUT"]
        else:
            env["CM_GIT_CHECKOUT"] = "master"

    if env.get('CM_GIT_URL', '') == '':
        if env.get('CM_TMP_GIT_URL', '') != '':
            env["CM_GIT_URL"] = env["CM_TMP_GIT_URL"]
        else:
            env["CM_GIT_URL"] = "https://github.com/mlcommons/inference"

    if env.get("CM_MLPERF_LAST_RELEASE", '') == '':
        env["CM_MLPERF_LAST_RELEASE"] = "v5.0"

    if 'CM_GIT_DEPTH' not in env:
        env['CM_GIT_DEPTH'] = ''

    if 'CM_GIT_RECURSE_SUBMODULES' not in env:
        env['CM_GIT_RECURSE_SUBMODULES'] = ''
    submodules = []
    possible_submodules = {
        "gn": "third_party/gn",
        "pybind": "third_party/pybind",
        "deeplearningexamples": "language/bert/DeepLearningExamples",
        "3d-unet": "vision/medical_imaging/3d-unet-brats19/nnUnet"
    }
    for submodule in possible_submodules:
        env_name = submodule.upper().replace("-", "_")
        if env.get("CM_SUBMODULE_" + env_name) == "yes":
            submodules.append(possible_submodules[submodule])

    env['CM_GIT_SUBMODULES'] = ",".join(submodules)

    if env.get('CM_GIT_PATCH_FILENAME', '') != '':
        patch_file_name = env['CM_GIT_PATCH_FILENAME']
        env['CM_GIT_PATCH_FILEPATHS'] = os.path.join(
            script_path, 'patch', patch_file_name)

    need_version = env.get('CM_VERSION', '')
    versions = meta['versions']

    if need_version != '' and not need_version in versions:
        env['CM_GIT_CHECKOUT'] = need_version

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    inference_root = env['CM_MLPERF_INFERENCE_SOURCE']
    env['CM_MLPERF_INFERENCE_VISION_PATH'] = os.path.join(
        inference_root, 'vision')
    env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'] = os.path.join(
        inference_root, 'vision', 'classification_and_detection')
    env['CM_MLPERF_INFERENCE_BERT_PATH'] = os.path.join(
        inference_root, 'language', 'bert')
    env['CM_MLPERF_INFERENCE_GPTJ_PATH'] = os.path.join(
        inference_root, 'language', 'gpt-j')
    env['CM_MLPERF_INFERENCE_RNNT_PATH'] = os.path.join(
        inference_root, 'speech_recognition', 'rnnt')
    env['CM_MLPERF_INFERENCE_DLRM_PATH'] = os.path.join(
        inference_root, 'recommendation', 'dlrm')
    env['CM_MLPERF_INFERENCE_DLRM_V2_PATH'] = os.path.join(
        inference_root, 'recommendation', 'dlrm_v2')
    env['CM_MLPERF_INFERENCE_RGAT_PATH'] = os.path.join(
        inference_root, 'graph', 'R-GAT')
    env['CM_MLPERF_INFERENCE_3DUNET_PATH'] = os.path.join(
        inference_root, 'vision', 'medical_imaging', '3d-unet-kits19')

    env['CM_GET_DEPENDENT_CACHED_PATH'] = inference_root

#        20221024: we save and restore env in the main script and can clean env here for determinism
#    if '+PYTHONPATH' not in env: env['+PYTHONPATH'] = []
    env['+PYTHONPATH'] = []
    env['+PYTHONPATH'].append(
        os.path.join(
            env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'],
            'python'))

    if os.path.exists(os.path.join(inference_root, "loadgen", "VERSION.txt")):
        with open(os.path.join(inference_root, "loadgen", "VERSION.txt")) as f:
            version_info = f.read().strip()
        env['CM_MLPERF_INFERENCE_SOURCE_VERSION'] = version_info

    if env.get('CM_GET_MLPERF_IMPLEMENTATION_ONLY', '') == "yes":
        return {'return': 0}

    env['CM_MLPERF_INFERENCE_CONF_PATH'] = os.path.join(
        inference_root, 'mlperf.conf')
    env['+PYTHONPATH'].append(
        os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            'tools',
            'submission'))

    valid_models = get_valid_models(
        env['CM_MLPERF_LAST_RELEASE'],
        env['CM_MLPERF_INFERENCE_SOURCE'])

    state['CM_MLPERF_INFERENCE_MODELS'] = valid_models

    if env.get('CM_GIT_REPO_CURRENT_HASH', '') != '':
        env['CM_VERSION'] += "-git-" + env['CM_GIT_REPO_CURRENT_HASH']

    return {'return': 0, 'version': env['CM_VERSION']}


def get_valid_models(mlperf_version, mlperf_path):

    import sys

    submission_checker_dir = os.path.join(mlperf_path, "tools", "submission")

    sys.path.append(submission_checker_dir)

    if not os.path.exists(os.path.join(
            submission_checker_dir, "submission_checker.py")):
        shutil.copy(os.path.join(submission_checker_dir, "submission-checker.py"), os.path.join(submission_checker_dir,
                                                                                                "submission_checker.py"))

    import submission_checker as checker

    config = checker.MODEL_CONFIG

    valid_models = config[mlperf_version]["models"]

    return valid_models



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import cmind as cm
from cmind import utils

import os
import subprocess
import json
import shutil


def preprocess(i):

    env = i['env']

    cur_dir = os.getcwd()

    # Query cache for results dirs
    env_repo_tags = env.get('CM_IMPORT_TINYMLPERF_REPO_TAGS', '').strip()
    xtags = '' if env_repo_tags == '' else ',version-' + env_repo_tags

    r = cm.access({'action': 'find',
                   'automation': 'cache,541d6f712a6b464e',
                   'tags': 'get,repo,mlperf-tiny-results' + xtags})
    if r['return'] > 0:
        return r

    lst = r['list']

    if len(lst) == 0:
        return {'return': 1, 'error': 'no repository with TinyMLPerf results found'}

    for c in lst:
        path = os.path.join(c.path, 'repo')

        if os.path.isdir(path):
            meta = c.meta

            tags = meta['tags']

            version = ''
            for t in tags:
                if t.startswith('version-'):
                    version = 'v' + t[8:]
                    break

            # Run local script
            run_script_input = i['run_script_input']
            automation = i['automation']

            env['CM_TINYMLPERF_REPO_PATH'] = path
            env['CM_TINYMLPERF_CURRENT_DIR'] = cur_dir
            env['CM_TINYMLPERF_REPO_VERSION'] = version

            print('')
            print('Repo path: {}'.format(path))

            r = automation.run_native_script({'run_script_input': run_script_input,
                                              'env': env,
                                              'script_name': 'run_submission_checker'})
            if r['return'] > 0:
                return r

    return {'return': 0}


def postprocess(i):

    env = i['env']

    path = env['CM_TINYMLPERF_REPO_PATH']
    cur_dir = env['CM_TINYMLPERF_CURRENT_DIR']
    version = env['CM_TINYMLPERF_REPO_VERSION']

    for ext in ['.csv', '.xlsx']:

        p1 = os.path.join(path, 'summary' + ext)
        p2 = os.path.join(cur_dir, 'summary-{}{}'.format(version, ext))

        if not os.path.isfile(p1):
            return {'return': 1, 'error': 'summary.csv file was not created'}

        if os.path.isfile(p2):
            os.remove(p2)

        shutil.copy(p1, p2)

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil
# used to measure the system infos(have not tested for obtaining gpu info)
import psutil
import csv         # used to write the measurements to csv format as txt file
from datetime import datetime, timezone
import time
import signal
import sys

# format of time measurement in mlperf logs
# :::MLLOG {"key": "power_begin", "value": "07-20-2024 17:54:38.800", "time_ms": 1580.314812, "namespace": "mlperf::logging", "event_type": "POINT_IN_TIME", "metadata": {"is_error": false, "is_warning": false, "file": "loadgen.cc", "line_no": 564, "pid": 9473, "tid": 9473}}
# :::MLLOG {"key": "power_end", "value": "07-20-2024 17:54:39.111", "time_ms": 1580.314812, "namespace": "mlperf::logging", "event_type": "POINT_IN_TIME", "metadata": {"is_error": false, "is_warning": false, "file": "loadgen.cc", "line_no": 566, "pid": 9473, "tid": 9473}}

# inorder to safely close when recieving interrupt signal
# argument sig: signal number
# argument frame: current stack frame


def signal_handler(sig, frame):
    print("Signal received, closing the system information file safely.")
    f.close()
    sys.exit(0)


# Register signal handlers for SIGTERM
signal.signal(signal.SIGTERM, signal_handler)


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    env = i['env']

    if env.get("CM_RUN_DIR", "") == "":
        env['CM_RUN_DIR'] = os.getcwd()

    logs_dir = env.get('CM_LOGS_DIR', env['CM_RUN_DIR'])

    log_json_file_path = os.path.join(logs_dir, 'sys_utilisation_info.txt')

    interval = int(env.get('CM_SYSTEM_INFO_MEASUREMENT_INTERVAL', '2'))

    print(f"The system dumps are created to the folder:{logs_dir}")

    print("WARNING: Currently the script is in its development stage. Only memory measurements supports as of now!")

    print("Started measuring system info!")

    csv_headers = [
        'timestamp',
        'cpu_utilisation',
        'total_memory_gb',
        'used_memory_gb']

    # done to be made available to signal_handler function in case of kill signals
    # as of now handles for only SIGTERM
    global f
    while True:
        with open(log_json_file_path, 'a', newline='') as f:
            writer = csv.DictWriter(f, fieldnames=csv_headers)
            # If the file is empty, write headers
            if f.tell() == 0:
                writer.writeheader()

            memory = psutil.virtual_memory()
            cpu_util = psutil.cpu_percent(interval=0)
            total_memory_gb = memory.total / (1024 ** 3)
            used_memory_gb = memory.used / (1024 ** 3)

            data = {
                'timestamp': datetime.now(timezone.utc).isoformat(),
                'cpu_utilisation': cpu_util,
                'total_memory_gb': total_memory_gb,
                'used_memory_gb': used_memory_gb
            }

            # Write data as a row to CSV file
            writer.writerow(data)
            time.sleep(interval)
            f.close()

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
from os.path import exists


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    dockerfile_path = env.get('CM_DOCKERFILE_WITH_PATH', '')
    if dockerfile_path != '' and os.path.exists(dockerfile_path):
        build_dockerfile = False
        env['CM_BUILD_DOCKERFILE'] = "no"
        os.chdir(os.path.dirname(dockerfile_path))
    else:
        build_dockerfile = True
        env['CM_BUILD_DOCKERFILE'] = "yes"
        env['CM_DOCKERFILE_BUILD_FROM_IMAGE_SCRIPT'] = "yes"

    CM_DOCKER_BUILD_ARGS = env.get('+ CM_DOCKER_BUILD_ARGS', [])

    if env.get('CM_GH_TOKEN', '') != '':
        CM_DOCKER_BUILD_ARGS.append("CM_GH_TOKEN=" + env['CM_GH_TOKEN'])

    if CM_DOCKER_BUILD_ARGS:
        build_args = "--build-arg " + \
            " --build-arg ".join(CM_DOCKER_BUILD_ARGS)
    else:
        build_args = ""

    env['CM_DOCKER_BUILD_ARGS'] = build_args

#    if 'CM_DOCKERFILE_WITH_PATH' not in env or not exists(env['CM_DOCKERFILE_WITH_PATH']):
#        env['CM_BUILD_DOCKERFILE'] = "yes"
#    else:
#        env['CM_BUILD_DOCKERFILE'] = "no"
#
    if env.get("CM_DOCKER_IMAGE_REPO", "") == '':
        env['CM_DOCKER_IMAGE_REPO'] = "localhost/local"

    docker_image_name = env.get('CM_DOCKER_IMAGE_NAME', '')
    if docker_image_name == '':
        docker_image_name = "cm-script-" + \
            env.get('CM_DOCKER_RUN_SCRIPT_TAGS', '').replace(
                ',', '-').replace('_', '-')

    env['CM_DOCKER_IMAGE_NAME'] = docker_image_name.lower()

    if env.get("CM_DOCKER_IMAGE_TAG", "") == '':
        env['CM_DOCKER_IMAGE_TAG'] = "latest"

    if str(env.get("CM_DOCKER_CACHE", "yes")).lower() in ["no", "false", "0"]:
        env["CM_DOCKER_CACHE_ARG"] = " --no-cache"

    CMD = ''

    image_name = get_image_name(env)

    if build_dockerfile:
        dockerfile_path = r"\${CM_DOCKERFILE_WITH_PATH}"

    # Write .dockerignore
    with open('.dockerignore', 'w') as f:
        f.write('.git\n')

    # Prepare CMD to build image
    XCMD = [
        f'{env["CM_CONTAINER_TOOL"]} build ' +
        env.get('CM_DOCKER_CACHE_ARG', ''),
        ' ' + build_args,
        ' -f "' + dockerfile_path + '"',
        ' -t "' + image_name,
        ' .'
    ]

    with open(dockerfile_path + '.build.sh', 'w') as f:
        f.write(' \\\n'.join(XCMD) + '\n')

    with open(dockerfile_path + '.build.bat', 'w') as f:
        f.write(' ^\n'.join(XCMD) + '\n')

    CMD = ''.join(XCMD)

    print('================================================')
    print('CM generated the following Docker build command:')
    print('')
    print(CMD)

    print('')

    env['CM_DOCKER_BUILD_CMD'] = CMD

    return {'return': 0}


def get_image_name(env):

    image_name = env.get('CM_DOCKER_IMAGE_REPO', '') + '/' + \
        env.get('CM_DOCKER_IMAGE_NAME', '') + ':' + \
        env.get('CM_DOCKER_IMAGE_TAG', '') + '"'

    return image_name


def postprocess(i):

    env = i['env']

    # Check if need to push docker image to the Docker Hub
    if env.get('CM_DOCKER_PUSH_IMAGE', '') in ['True', True, 'yes']:
        image_name = get_image_name(env)

        # Prepare CMD to build image
        PCMD = 'docker image push ' + image_name

        dockerfile_path = env.get('CM_DOCKERFILE_WITH_PATH', '')
        if dockerfile_path != '' and os.path.isfile(dockerfile_path):
            with open(dockerfile_path + '.push.sh', 'w') as f:
                f.write(PCMD + '\n')

            with open(dockerfile_path + '.build.bat', 'w') as f:
                f.write(PCMD + '\n')

        print('================================================')
        print('CM generated the following Docker push command:')
        print('')
        print(PCMD)

        print('')

        r = os.system(PCMD)
        print('')

        if r > 0:
            return {'return': 1, 'error': 'pushing to Docker Hub failed'}

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import cmind as cm
from cmind import utils

import os
import subprocess
import csv
import json
import copy


file_summary = 'summary.csv'
file_summary_json = 'mlperf-inference-summary-{}.json'
file_result = 'cm-result.json'

model2task = {
    "resnet": "image-classification",
    "retinanet": "object-detection",
    "ssd-small": "object-detection",
    "ssd-large": "object-detection",
    "rnnt": "speech-recognition",
    "bert-99": "language-processing",
    "bert-99.9": "language-processing",
    "gptj-99": "language-processing",
    "gptj-99.9": "language-processing",
    "llama2-70b-99": "language-processing",
    "llama2-70b-99.9": "language-processing",
    "dlrm-99": "recommendation",
    "dlrm-v2-99": "recommendation",
    "dlrm-99.9": "recommendation",
    "dlrm-v2-99.9": "recommendation",
    "3d-unet-99": "image-segmentation",
    "3d-unet-99.9": "image-segmentation",
    "stable-diffusion-xl": "text-to-image"
}


def preprocess(i):

    env = i['env']

    cur_dir = os.getcwd()

    # Query cache for results dirs
    r = cm.access({'action': 'find',
                   'automation': 'cache,541d6f712a6b464e',
                   'tags': 'get,repo,mlperf-inference-results'})
    if r['return'] > 0:
        return r

    lst = r['list']

    for c in lst:
        path = os.path.join(c.path, 'repo')

        if os.path.isdir(path):

            meta = c.meta

            tags = meta['tags']

            version = ''
            for t in tags:
                if t.startswith('version-'):
                    version = 'v' + t[8:]
                    break

            skip_submission_checker = env.get(
                'CM_SKIP_SUBMISSION_CHECKER', '') in [
                'yes', 'True']

            print('')
            print('Processing results in path: {}'.format(path))
            print('Version: {}'.format(version))
            print('')

            if skip_submission_checker:
                if not os.path.isfile(file_summary):
                    return {'return': 1,
                            'error': '{} not found'.format(file_summary)}
            else:
                if os.path.isfile(file_summary):
                    os.remove(file_summary)

                print('* Running submission checker ...')

                xenv = {}

                submitter = env.get('CM_MLPERF_SUBMITTER', '')
                if submitter != '':
                    xenv['CM_MLPERF_SUBMITTER'] = submitter

                ii = {'action': 'run',
                      'automation': 'script',
                      'tags': 'run,mlperf,inference,submission,checker',
                      'extra_args': ' --skip-extra-files-in-root-check',
                      'submission_dir': path}

                if len(xenv) > 0:
                    ii['env'] = xenv

                if version != '':
                    print(
                        '  Version detected from cache tags: {}'.format(version))
                    ii['version'] = version

                r = cm.access(ii)
                # Ignore if script fails for now (when some results are wrong)
                if r['return'] > 0 and r['return'] != 2:
                    return r

                if r['return'] > 0:
                    print('')
                    print(
                        'WARNING: script returned non-zero value - possible issue - please check!')
                    print('')
                    input('Press Enter to continue')
                    print('')

            r = convert_summary_csv_to_experiment(path, version, env)
            if r['return'] > 0:
                return r

    return {'return': 0}


def convert_summary_csv_to_experiment(path, version, env):
    print('* Processing MLPerf repo in cache path: {}'.format(path))

    cur_dir = os.getcwd()

    # Get Git URL
    os.chdir(path)

    burl = subprocess.check_output(
        ['git', 'config', '--get', 'remote.origin.url'])
    url = burl.decode('UTF-8').strip()

    print('  Git URL: {}'.format(url))

    os.chdir(cur_dir)

    if os.path.isfile(file_summary):
        summary = []

        with open(file_summary, encoding='utf-8') as fcsv:
            csv_reader = csv.DictReader(fcsv)

            for rows in csv_reader:
                result = {}

                keys = rows.keys()

                for k in keys:
                    v = rows[k]

                    if v == 'False':
                        v = False
                    elif v == 'True':
                        v = True
                    else:
                        try:
                            v = float(v)

                            if v == int(v):
                                v = int(v)
                        except ValueError:
                            pass

                    result[k] = v

                # Add extra tags
                if url != '':
                    result['git_url'] = url

                    location = result.get('Location', '')
                    if location != '':
                        result['url'] = url + '/tree/master/' + location

                accuracy = result.get('Accuracy', 0.0)
#
#                print (accuracy, type(accuracy))
                if accuracy is not None and accuracy != 'None' and accuracy > 0:
                    result['Accuracy_div_100'] = float(
                        '{:.5f}'.format(result['Accuracy'] / 100))

                # Add ratios

                # Append to summary
                summary.append(result)

        r = utils.save_json(file_summary_json.format(version), summary)
        if r['return'] > 0:
            return r

        # Create virtual experiment entries
        experiment = {}

        for result in summary:

            # Create name
            mlperfmodel = result['MlperfModel']
            task = model2task[mlperfmodel]

            system_type = result['SystemType']

            division = result['Division']
            has_power = result.get('has_power', False)

            if division == 'network':
                xdivision = 'closed-network'
            else:
                xdivision = division.lower()
                if has_power:
                    xdivision += '-power'

            # If datacenter,edge - remove ,edge to be consistent with
            # https://mlcommons.org/en/inference-datacenter-21/
            j = system_type.find(',')
            if j >= 0:
                system_type = system_type[:j]

            scenario = result['Scenario'].lower()

            name = 'mlperf-inference--{}--' + system_type + \
                '--' + xdivision + '--' + task + '--' + scenario

            name_all = name.format('all')
            name_ver = name.format(version)

            for name in [name_all, name_ver]:
                if name not in experiment:
                    experiment[name] = []
                experiment[name].append(result)

        # Checking experiment
        env_target_repo = env.get(
            'CM_IMPORT_MLPERF_INFERENCE_TARGET_REPO', '').strip()
        target_repo = '' if env_target_repo == '' else env_target_repo + ':'

        print('')
        for name in experiment:
            print('    Preparing experiment artifact "{}"'.format(name))

            tags = name.split('--')
            if 'mlperf' not in tags:
                tags.insert(0, 'mlperf')

            # Checking if experiment already exists
            r = cm.access({'action': 'find',
                           'automation': 'experiment,a0a2d123ef064bcb',
                           'artifact': target_repo + name})
            if r['return'] > 0:
                return r

            lst = r['list']

            if len(lst) == 0:
                r = cm.access({'action': 'add',
                               'automation': 'experiment,a0a2d123ef064bcb',
                               'artifact': target_repo + name,
                               'tags': tags})
                if r['return'] > 0:
                    return r

                path = r['path']
            else:
                path = lst[0].path

            results = experiment[name]

            # Check if already date directory
            dirs = os.listdir(path)

            path2 = ''
            for d in dirs:
                dd = os.path.join(path, d)
                if os.path.isdir(dd):
                    path2 = dd
                    break

            if path2 == '':

                r = utils.get_current_date_time({})
                if r['return'] > 0:
                    return r

                date_time = r['iso_datetime'].replace(
                    ':', '-').replace('T', '.')

                path2 = os.path.join(path, date_time)

                os.makedirs(path2)

            # Check if cm-result.json
            fresult = os.path.join(path2, file_result)

            if os.path.isfile(fresult):
                r = utils.load_json(fresult)
                if r['return'] > 0:
                    return r

                existing_results = r['meta']

                # Need to check which ones to add
                for result in existing_results:
                    found = False

                    # New results
                    for result2 in results:
                        matched = True

                        # Need to iterate over keys in the new results since
                        # old results can have more keys (derivates, etc)
                        for k in result2:
                            if k != 'uid':
                                if k not in result or result2[k] != result[k]:
                                    matched = False
                                    break

                        if matched:
                            found = True
                            break

                    if not found:
                        results.append(result)

            # Check extra keys
            final_results = []
            for result in results:
                # Generate UID
                if 'uid' not in result:
                    r = utils.gen_uid()
                    if r['return'] > 0:
                        return r

                    result['uid'] = r['uid']

                # Get Result and Units together
                if 'Result' in result and 'Units' in result:
                    result['Result_Units'] = result['Units']

                # Temporal hack for Power to separate power from the graph
                units = result.get('Units', '')
                if units == 'Watts' or 'joules' in units:
                    if 'Result_Power' not in result:
                        result['Result_Power'] = result['Result']
                        result['Result'] = None

            # Write results
            r = utils.save_json(fresult, results)
            if r['return'] > 0:
                return r

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    code_path = os.path.join(
        env['CM_GIT_REPO_CHECKOUT_PATH'],
        "closed",
        "fpgaconvnet",
        "code")
    network_env_name = env['CM_TINY_NETWORK_NAME'].replace("-", "_").upper()
    env['CM_TINY_FPGACONVNET_NETWORK_ENV_NAME'] = network_env_name
    env['CM_TINY_FPGACONVNET_' + network_env_name + '_CODE_PATH'] = code_path

    board = env.get('CM_TINY_BOARD', 'zc706')

    benchmark = env.get('CM_TINY_BENCHMARK', 'ic')

    run_dir = os.path.join(code_path, board, benchmark)
    env['CM_TINY_FPGACONVNET_' + network_env_name + '_RUN_DIR'] = run_dir

    run_cmd = "cd " + run_dir + " && " + \
        env['CM_PYTHON_BIN_WITH_PATH'] + " " + "create_config.py"

    env['ML_MODEL_FILE_WITH_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']
    env['CM_RUN_CMD'] = run_cmd
    env['CM_RUN_DIR'] = run_dir

    return {'return': 0}


def postprocess(i):

    env = i['env']

    network = env['CM_TINY_NETWORK_NAME']
    env['CM_TINY_FPGACONVNET_NETWORK_NAME'] = network
    network_env_name = env['CM_TINY_FPGACONVNET_NETWORK_ENV_NAME']

    json_location = os.path.join(
        env['CM_RUN_DIR'],
        env['CM_TINY_NETWORK_NAME'] + ".json")
    if os.path.exists(json_location):
        print(
            f"JSON configuration file for {network} created at {json_location}")
    else:
        return {'return': 1, 'error': "JSON configuration file generation failed"}

    env['CM_TINY_FPGACONVNET_CONFIG_FILE_' +
        network_env_name + '_PATH'] = json_location
    env['CM_GET_DEPENDENT_CACHED_PATH'] = json_location

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']
    cm = automation.cmind

    if env.get('CM_HOST_OS_FLAVOR', '') == 'amzn':
        env['CM_PACKAGE_TOOL'] = "yum"
        i['run_script_input']['script_name'] = "run-rhel"

    # Test (not needed - will be removed)
    if str(env.get('CM_SKIP_SYS_UTILS', '')).lower() in [True, 'yes', 'on']:
        return {'return': 0, 'skip': True}


# Windows has moved to get-sys-utils-min and will be always run with
# "detect,os"!

    if os_info['platform'] == 'windows':
        print('')
        print('This script is not used on Windows')
        print('')

   # If windows, download here otherwise use run.sh

#
#        path = os.getcwd()
#
#        clean_dirs = env.get('CM_CLEAN_DIRS','').strip()
#        if clean_dirs!='':
#            import shutil
#            for cd in clean_dirs.split(','):
#                if cd != '':
#                    if os.path.isdir(cd):
#                        print ('Clearning directory {}'.format(cd))
#                        shutil.rmtree(cd)
#
#        url = env['CM_PACKAGE_WIN_URL']
#
#        urls = [url] if ';' not in url else url.split(';')
#
#        print ('')
#        print ('Current directory: {}'.format(os.getcwd()))
#
#        for url in urls:
#
#            url = url.strip()
#
#            print ('')
#            print ('Downloading from {}'.format(url))
#
#            r = cm.access({'action':'download_file',
#                           'automation':'utils,dc2743f8450541e3',
#                           'url':url})
#            if r['return']>0: return r
#
#            filename = r['filename']
#
#            print ('Unzipping file {}'.format(filename))
#
#            r = cm.access({'action':'unzip_file',
#                           'automation':'utils,dc2743f8450541e3',
#                           'filename':filename})
#            if r['return']>0: return r
#
#            if os.path.isfile(filename):
#                print ('Removing file {}'.format(filename))
#                os.remove(filename)
#
#        print ('')
#
#        # Add to path
#        env['+PATH']=[os.path.join(path, 'bin')]
#
    else:
        print('')
        print('***********************************************************************')
        print('This script will attempt to install minimal system dependencies for CM.')
        print('Note that you may be asked for your SUDO password ...')
        print('***********************************************************************')

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if env.get('CM_TMP_ML_MODEL_RETINANET_NO_NMS', '') == 'yes':
        i['run_script_input']['script_name'] = "run-no-nms"
        env['CM_ML_MODEL_FILE_WITH_PATH'] = os.path.join(
            os.getcwd(), "retinanet.onnx")

    return {'return': 0}


def postprocess(i):

    env = i['env']

    env['CM_ML_MODEL_FILE'] = os.path.basename(
        env['CM_ML_MODEL_FILE_WITH_PATH'])
    if env.get('CM_ENV_NAME_ML_MODEL_FILE', '') != '':
        env[env['CM_ENV_NAME_ML_MODEL_FILE']] = env['CM_ML_MODEL_FILE_WITH_PATH']

    if env.get("CM_QAIC_PRINT_NODE_PRECISION_INFO", '') == 'yes':
        env['CM_ML_MODEL_RETINANET_QAIC_NODE_PRECISION_INFO_FILE_PATH'] = os.path.join(
            os.getcwd(), 'node-precision-info.yaml')

    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import subprocess
from os.path import exists
import json


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    interactive = env.get('CM_DOCKER_INTERACTIVE_MODE', '')

    if str(interactive).lower() in ['yes', 'true', '1']:
        env['CM_DOCKER_DETACHED_MODE'] = 'no'

    if 'CM_DOCKER_RUN_SCRIPT_TAGS' not in env:
        env['CM_DOCKER_RUN_SCRIPT_TAGS'] = "run,docker,container"
        CM_RUN_CMD = "cm version"
    else:
        CM_RUN_CMD = "cm run script --tags=" + \
            env['CM_DOCKER_RUN_SCRIPT_TAGS'] + ' --quiet'

    r = cm.access({'action': 'search',
                   'automation': 'script',
                   'tags': env['CM_DOCKER_RUN_SCRIPT_TAGS']})
    if len(r['list']) < 1:
        raise Exception(
            'CM script with tags ' +
            env['CM_DOCKER_RUN_SCRIPT_TAGS'] +
            ' not found!')

    PATH = r['list'][0].path
    os.chdir(PATH)

    env['CM_DOCKER_RUN_CMD'] = CM_RUN_CMD

    # Updating Docker info
    update_docker_info(env)

    docker_image_repo = env['CM_DOCKER_IMAGE_REPO']
    docker_image_base = env['CM_DOCKER_IMAGE_BASE']
    docker_image_name = env['CM_DOCKER_IMAGE_NAME']
    docker_image_tag = env['CM_DOCKER_IMAGE_TAG']

    DOCKER_CONTAINER = docker_image_repo + "/" + \
        docker_image_name + ":" + docker_image_tag

    print('')
    print('Checking existing Docker container:')
    print('')
    CMD = f"""{env['CM_CONTAINER_TOOL']} ps --format=json  --filter "ancestor={DOCKER_CONTAINER}" """
    if os_info['platform'] == 'windows':
        CMD += " 2> nul"
    else:
        CMD += " 2> /dev/null"
    print('  ' + CMD)
    print('')

    try:
        out = subprocess.check_output(
            CMD, shell=True, text=True).strip()
    except Exception as e:
        return {
            'return': 1,
            'error': 'Unexpected error occurred with docker run:\n{}'.format(e)
        }

    if len(out) > 0 and str(env.get('CM_DOCKER_REUSE_EXISTING_CONTAINER',
                                    '')).lower() in ["1", "true", "yes"]:  # container exists
        # print(out)
        out_split = out.splitlines()
        if len(out_split) > 0:
            try:
                out_json = json.loads(out_split[0])
                # print("JSON successfully loaded:", out_json)
            except json.JSONDecodeError as e:
                print(f"Error: First line of 'out' is not valid JSON: {e}")
                return {
                    'return': 1, 'error': f"Error: First line of 'out' is not valid JSON: {e}"}
    else:
        out_json = []

    if isinstance(out_json, list) and len(out_json) > 0:
        existing_container_id = out_json[0]['Id']
        print(f"Reusing existing container {existing_container_id}")
        env['CM_DOCKER_CONTAINER_ID'] = existing_container_id

    else:
        if env.get('CM_DOCKER_CONTAINER_ID', '') != '':
            del (env['CM_DOCKER_CONTAINER_ID'])  # not valid ID

        CMD = f"""{env['CM_CONTAINER_TOOL']} images -q """ + DOCKER_CONTAINER

        if os_info['platform'] == 'windows':
            CMD += " 2> nul"
        else:
            CMD += " 2> /dev/null"

        print('')
        print('Checking Docker images:')
        print('')
        print('  ' + CMD)
        print('')

        try:
            docker_image = subprocess.check_output(
                CMD, shell=True).decode("utf-8")
        except Exception as e:
            return {
                'return': 1, 'error': 'Docker is either not installed or not started:\n{}'.format(e)}

        recreate_image = env.get('CM_DOCKER_IMAGE_RECREATE', '')

        if recreate_image != 'yes':
            if docker_image:
                print("Docker image exists with ID: " + docker_image)
                env['CM_DOCKER_IMAGE_EXISTS'] = "yes"

    #    elif recreate_image == "yes":
    #        env['CM_DOCKER_IMAGE_RECREATE'] = "no"

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']

    # Updating Docker info
    update_docker_info(env)

    docker_image_repo = env['CM_DOCKER_IMAGE_REPO']
    docker_image_base = env['CM_DOCKER_IMAGE_BASE']
    docker_image_name = env['CM_DOCKER_IMAGE_NAME']
    docker_image_tag = env['CM_DOCKER_IMAGE_TAG']

    run_cmds = []
    mount_cmds = []
    port_map_cmds = []
    run_opts = ''

    # not completed as su command breaks the execution sequence
    #
    # if env.get('CM_DOCKER_PASS_USER_ID', '') != '':
    #    run_opts += " --user 0 "
    #    run_cmds.append(f"(usermod -u {os.getuid()} cmuser || echo pass)")
    #    run_cmds.append(f"(chown -R {os.getuid()}:{os.getuid()} /home/cmuser  || echo pass)")
    #    run_cmds.append(" ( su cmuser )")
    #    run_cmds.append('export PATH="/home/cmuser/venv/cm/bin:$PATH"')

    if env.get('CM_DOCKER_PRE_RUN_COMMANDS', []):
        for pre_run_cmd in env['CM_DOCKER_PRE_RUN_COMMANDS']:
            run_cmds.append(pre_run_cmd)

    if env.get('CM_DOCKER_VOLUME_MOUNTS', []):
        for mounts in env['CM_DOCKER_VOLUME_MOUNTS']:
            mount_cmds.append(mounts)

    if env.get('CM_DOCKER_PASS_USER_GROUP', '') != '':
        run_opts += " --group-add $(id -g $USER) "

    if env.get('CM_DOCKER_ADD_DEVICE', '') != '':
        run_opts += " --device=" + env['CM_DOCKER_ADD_DEVICE']

    if env.get('CM_DOCKER_PRIVILEGED_MODE', '') == 'yes':
        run_opts += " --privileged "

    if env.get('CM_DOCKER_ADD_NUM_GPUS', '') != '':
        run_opts += " --gpus={}".format(env['CM_DOCKER_ADD_NUM_GPUS'])
    elif env.get('CM_DOCKER_ADD_ALL_GPUS', '') != '':
        run_opts += " --gpus=all"

    if env.get('CM_DOCKER_SHM_SIZE', '') != '':
        run_opts += " --shm-size={}".format(env['CM_DOCKER_SHM_SIZE'])

    if env.get('CM_DOCKER_EXTRA_RUN_ARGS', '') != '':
        run_opts += env['CM_DOCKER_EXTRA_RUN_ARGS']

    if env.get('CM_CONTAINER_TOOL', '') == 'podman' and env.get(
            'CM_PODMAN_MAP_USER_ID', '').lower() not in ["no", "0", "false"]:
        run_opts += " --userns=keep-id"

    if env.get('CM_DOCKER_PORT_MAPS', []):
        for ports in env['CM_DOCKER_PORT_MAPS']:
            port_map_cmds.append(ports)

    run_cmd = env['CM_DOCKER_RUN_CMD'] + " " + \
        env.get('CM_DOCKER_RUN_CMD_EXTRA', '').replace(":", "=")
    run_cmds.append(run_cmd)
    if 'CM_DOCKER_POST_RUN_COMMANDS' in env:
        for post_run_cmd in env['CM_DOCKER_POST_RUN_COMMANDS']:
            run_cmds.append(post_run_cmd)

    run_cmd = " && ".join(run_cmds)
    run_cmd = run_cmd.replace("--docker_run_deps", "")

    if mount_cmds:
        for i, mount_cmd in enumerate(mount_cmds):

            # Since windows may have 2 :, we search from the right
            j = mount_cmd.rfind(':')
            if j > 0:
                mount_parts = [mount_cmd[:j], mount_cmd[j + 1:]]
            else:
                return {'return': 1, 'error': 'Can\'t find separator : in a mount string: {}'.format(
                    mount_cmd)}

            host_mount = mount_parts[0]

            if not os.path.exists(host_mount):
                os.makedirs(host_mount)

            abs_host_mount = os.path.abspath(mount_parts[0])

            if abs_host_mount != host_mount or " " in abs_host_mount and not host_mount.startswith(
                    '"'):
                mount_cmds[i] = f"\"{abs_host_mount}\":{mount_parts[1]}"

        mount_cmd_string = " -v " + " -v ".join(mount_cmds)
    else:
        mount_cmd_string = ''
    run_opts += mount_cmd_string

    if port_map_cmds:
        port_map_cmd_string = " -p " + "-p ".join(port_map_cmds)
    else:
        port_map_cmd_string = ''

    run_opts += port_map_cmd_string

    # Currently have problem running Docker in detached mode on Windows:
    detached = str(
        env.get(
            'CM_DOCKER_DETACHED_MODE',
            '')).lower() in [
        'yes',
        'true',
        "1"]
#    if detached and os_info['platform'] != 'windows':
    if detached:
        if os_info['platform'] == 'windows':
            return {
                'return': 1, 'error': 'Currently we don\'t support running Docker containers in detached mode on Windows - TBD'}

        existing_container_id = env.get('CM_DOCKER_CONTAINER_ID', '')
        if existing_container_id:
            CMD = f"""ID={existing_container_id} && {env['CM_CONTAINER_TOOL']} exec $ID bash -c '""" + run_cmd + "'"
        else:
            CONTAINER = f"""{env['CM_CONTAINER_TOOL']} run -dt {run_opts} --rm  {docker_image_repo}/{docker_image_name}:{docker_image_tag} bash"""
            CMD = f"""ID=`{CONTAINER}` && {env['CM_CONTAINER_TOOL']} exec $ID bash -c '{run_cmd}'"""

            if False and str(env.get('CM_KEEP_DETACHED_CONTAINER', '')).lower() not in [
                    'yes', "1", 'true']:
                CMD += f""" && {env['CM_CONTAINER_TOOL']} kill $ID >/dev/null"""

        CMD += ' && echo "ID=$ID"'

        print('=========================')
        print("Container launch command:")
        print('')
        print(CMD)
        print('')
        print(
            "Running " +
            run_cmd +
            f""" inside {env['CM_CONTAINER_TOOL']} container""")

        record_script({'cmd': CMD, 'env': env})

        print('')
        # Execute the command
        try:
            result = subprocess.run(
                CMD,
                shell=True,
                check=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True)
            print("Command Output:", result.stdout)
        except subprocess.CalledProcessError as e:
            print("Error Occurred!")
            print(f"Command: {e.cmd}")
            print(f"Return Code: {e.returncode}")
            print(f"Error Output: {e.stderr}")
            return {'return': 1, 'error': e.stderr}

        docker_out = result.stdout
        # if docker_out != 0:
        # return {'return': docker_out, 'error': f""{env['CM_CONTAINER_TOOL']}
        # run failed""}

        lines = docker_out.split("\n")

        for line in lines:
            if line.startswith("ID="):
                ID = line[3:]
                env['CM_DOCKER_CONTAINER_ID'] = ID

        print(docker_out)

    else:
        x = "'"
        if os_info['platform'] == 'windows':
            x = '"'

        x1 = ''
        x2 = ''
        run_cmd_prefix = ""
        if env.get('CM_DOCKER_INTERACTIVE_MODE', '') in ['yes', 'True', True]:
            run_cmd_prefix = "("
            x1 = '-it'
            x2 = " && bash ) || bash"

        CONTAINER = f"{env['CM_CONTAINER_TOOL']} run " + x1 + " --entrypoint " + x + x + " " + run_opts + \
            " " + docker_image_repo + "/" + docker_image_name + ":" + docker_image_tag
        CMD = CONTAINER + " bash -c " + x + run_cmd_prefix + run_cmd + x2 + x

        print('')
        print("Container launch command:")
        print('')
        print(CMD)

        record_script({'cmd': CMD, 'env': env})

        print('')
        docker_out = os.system(CMD)
        if docker_out != 0:
            if docker_out % 256 == 0:
                docker_out = 1
            return {'return': docker_out,
                    'error': f"""{env['CM_CONTAINER_TOOL']} run failed"""}

    return {'return': 0}


def record_script(i):

    cmd = i['cmd']
    env = i['env']

    files = []

    dockerfile_path = env.get('CM_DOCKERFILE_WITH_PATH', '')
    if dockerfile_path != '' and os.path.isfile(dockerfile_path):
        files.append(dockerfile_path + '.run.bat')
        files.append(dockerfile_path + '.run.sh')

    save_script = env.get('CM_DOCKER_SAVE_SCRIPT', '')
    if save_script != '':
        if save_script.endswith('.bat') or save_script.endswith('.sh'):
            files.append(save_script)
        else:
            files.append(save_script + '.bat')
            files.append(save_script + '.sh')

    for filename in files:
        with open(filename, 'w') as f:
            f.write(cmd + '\n')

    return {'return': 0}


def update_docker_info(env):

    # Updating Docker info
    docker_image_repo = env.get('CM_DOCKER_IMAGE_REPO', 'localhost/local')
    env['CM_DOCKER_IMAGE_REPO'] = docker_image_repo

    docker_image_base = env.get('CM_DOCKER_IMAGE_BASE')
    if not docker_image_base:
        if env.get("CM_DOCKER_OS", '') != '':
            docker_image_base = env["CM_DOCKER_OS"] + \
                ":" + env["CM_DOCKER_OS_VERSION"]
        else:
            docker_image_base = "ubuntu:22.04"

    env['CM_DOCKER_IMAGE_BASE'] = docker_image_base

    if env.get('CM_DOCKER_IMAGE_NAME', '') != '':
        docker_image_name = env['CM_DOCKER_IMAGE_NAME']
    else:
        docker_image_name = 'cm-script-' + \
            env['CM_DOCKER_RUN_SCRIPT_TAGS'].replace(
                ',', '-').replace('_', '-').replace('+', 'plus')
        env['CM_DOCKER_IMAGE_NAME'] = docker_image_name

    docker_image_tag_extra = env.get('CM_DOCKER_IMAGE_TAG_EXTRA', '-latest')

    docker_image_tag = env.get('CM_DOCKER_IMAGE_TAG', docker_image_base.replace(
        ':', '-').replace('_', '').replace("/", "-") + docker_image_tag_extra)
    env['CM_DOCKER_IMAGE_TAG'] = docker_image_tag

    return



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    state = i['state']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    if not state.get(
            'mlperf-inference-implementation'):  # No state information. Just returning
        return {'return': 0}

    if env.get('CM_MLPERF_README', "") == "yes":
        import cmind as cm
        inp = i['input']

        script_tags = state['mlperf-inference-implementation'].get(
            'script_tags', '')
        script_adr = state['mlperf-inference-implementation'].get(
            'script_adr', {})

        if script_tags != '':
            cm_input = {'action': 'run',
                        'automation': 'script',
                        'tags': script_tags,
                        'adr': script_adr,
                        'env': env,
                        'print_deps': True,
                        'quiet': True,
                        'silent': True,
                        'fake_run': True
                        }

            r = cm.access(cm_input)
            if r['return'] > 0:
                return r

            state['mlperf-inference-implementation']['print_deps'] = r['new_state']['print_deps']

    if env.get('CM_DUMP_VERSION_INFO', True):

        if state['mlperf-inference-implementation'].get('script_id', '') == '':
            state['mlperf-inference-implementation']['script_id'] = ''

        script_id = state['mlperf-inference-implementation']['script_id']
        run_state = i['input']['run_state']
        version_info = {}
        version_info[script_id] = run_state['version_info']

        state['mlperf-inference-implementation']['version_info'] = version_info

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}
    env = i['env']

    if str(env.get('CM_RUN_STATE_DOCKER', '')).lower() in ['1', 'true', 'yes']:
        return {'return': 0}

    if env.get('CM_MODEL', '') == '':
        return {
            'return': 1, 'error': 'Please select a variation specifying the model to run'}

    make_command = env['MLPERF_NVIDIA_RUN_COMMAND']

    if env.get('CM_MLPERF_DEVICE', '') == '':
        return {
            'return': 1, 'error': 'Please select a variation specifying the device to run on'}

    if env.get('CM_MLPERF_SKIP_RUN',
               '') == "yes" and make_command == "run_harness":
        return {'return': 0}

    env['MLPERF_SCRATCH_PATH'] = env['CM_NVIDIA_MLPERF_SCRATCH_PATH']

    cmds = []
    scenario = env['CM_MLPERF_LOADGEN_SCENARIO']
    mode = env['CM_MLPERF_LOADGEN_MODE']

    make_command = env['MLPERF_NVIDIA_RUN_COMMAND']

    if make_command == "prebuild":
        cmds.append(f"""make prebuild NETWORK_NODE=SUT""")

    if env['CM_MODEL'] == "resnet50":
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'data', 'imagenet')
        if not os.path.exists(target_data_path):
            cmds.append(
                f"""ln -sf {env['CM_DATASET_IMAGENET_PATH']} {target_data_path}""")

        model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'ResNet50',
            'resnet50_v1.onnx')

        if not os.path.exists(os.path.dirname(model_path)):
            cmds.append(f"""mkdir -p {os.path.dirname(model_path)}""")

        if not os.path.exists(model_path):
            cmds.append(
                f"""ln -sf {env['CM_ML_MODEL_FILE_WITH_PATH']} {model_path}""")
        model_name = "resnet50"

    elif "bert" in env['CM_MODEL']:
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'data', 'squad')
        if not os.path.exists(target_data_path):
            cmds.append("make download_data BENCHMARKS='bert'")

        fp32_model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'bert',
            'bert_large_v1_1.onnx')
        int8_model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'bert',
            'bert_large_v1_1_fake_quant.onnx')
        vocab_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'bert',
            'vocab.txt')

        if not os.path.exists(os.path.dirname(fp32_model_path)):
            cmds.append(f"""mkdir -p {os.path.dirname(fp32_model_path)}""")

        if not os.path.exists(fp32_model_path):
            cmds.append(
                f"""cp -r --remove-destination {env['CM_ML_MODEL_BERT_LARGE_FP32_PATH']} {fp32_model_path}""")
        if not os.path.exists(int8_model_path):
            cmds.append(
                f"""cp -r --remove-destination {env['CM_ML_MODEL_BERT_LARGE_INT8_PATH']} {int8_model_path}""")
        if not os.path.exists(vocab_path):
            cmds.append(
                f"""cp -r --remove-destination {env['CM_ML_MODEL_BERT_VOCAB_FILE_WITH_PATH']} {vocab_path}""")
        model_name = "bert"
        model_path = fp32_model_path

    elif "stable-diffusion" in env["CM_MODEL"]:
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'data', 'coco', 'SDXL')
        tsv_file = os.path.join(target_data_path, "captions_5k_final.tsv")
        if os.path.exists(tsv_file):
            with open(tsv_file, "r") as file:
                line_count = sum(1 for line in file)
            if env.get('CM_MLPERF_SUBMISSION_GENERATION_STYLE', '') == 'full':
                if line_count < 5000:
                    shutil.rmtree(target_data_path)
        if not os.path.exists(tsv_file):
            os.makedirs(target_data_path, exist_ok=True)
            # cmds.append("make download_data BENCHMARKS='stable-diffusion-xl'")
            env['CM_REQUIRE_COCO2014_DOWNLOAD'] = 'yes'
            cmds.append(
                f"""cp -r \\$CM_DATASET_PATH_ROOT/captions/captions.tsv {target_data_path}/captions_5k_final.tsv""")
            cmds.append(
                f"""cp -r \\$CM_DATASET_PATH_ROOT/latents/latents.pt {target_data_path}/latents.pt""")
        fp16_model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'SDXL',
            'official_pytorch',
            'fp16',
            'stable_diffusion_fp16')

        if not os.path.exists(os.path.dirname(fp16_model_path)):
            cmds.append(f"""mkdir -p {os.path.dirname(fp16_model_path)}""")

        if not os.path.exists(fp16_model_path):
            if os.path.islink(fp16_model_path):
                cmds.append(f"rm -f {fp16_model_path}")
            env['CM_REQUIRE_SDXL_MODEL_DOWNLOAD'] = 'yes'
            cmds.append(f"cp -r \\$SDXL_CHECKPOINT_PATH {fp16_model_path}")

        model_name = "stable-diffusion-xl"
        model_path = fp16_model_path

    elif "3d-unet" in env['CM_MODEL']:
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'data',
            'KiTS19',
            'kits19',
            'data')
        target_data_path_base_dir = os.path.dirname(target_data_path)
        if not os.path.exists(target_data_path_base_dir):
            cmds.append(f"mkdir -p {target_data_path_base_dir}")

        inference_cases_json_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'data', 'KiTS19', 'inference_cases.json')
        calibration_cases_json_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'data', 'KiTS19', 'calibration_cases.json')

        if not os.path.exists(target_data_path) or not os.path.exists(
                inference_cases_json_path) or not os.path.exists(calibration_cases_json_path):
            # cmds.append(f"ln -sf {env['CM_DATASET_PATH']} {target_data_path}")
            cmds.append("make download_data BENCHMARKS='3d-unet'")

        model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            '3d-unet-kits19',
            '3dUNetKiTS19.onnx')
        model_name = "3d-unet"

    elif "rnnt" in env['CM_MODEL']:
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'data',
            'LibriSpeech',
            'dev-clean')
        target_data_path_base_dir = os.path.dirname(target_data_path)
        if not os.path.exists(target_data_path_base_dir):
            cmds.append(f"mkdir -p {target_data_path_base_dir}")
        if not os.path.exists(target_data_path):
            # cmds.append(f"ln -sf {env['CM_DATASET_LIBRISPEECH_PATH']} {target_data_path}")
            cmds.append("make download_data BENCHMARKS='rnnt'")

        model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'rnn-t',
            'DistributedDataParallel_1576581068.9962234-epoch-100.pt')
        model_name = "rnnt"

    elif "pdlrm" in env['CM_MODEL']:
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'data', 'criteo')
        if not os.path.exists(target_data_path):
            cmds.append(
                f"ln -sf {env['CM_DATASET_PREPROCESSED_PATH']} {target_data_path}")

        model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'dlrm',
            'tb00_40M.pt')
        if not os.path.exists(os.path.dirname(model_path)):
            cmds.append(f"mkdir -p {os.path.dirname(model_path)}")

        if not os.path.exists(model_path):
            cmds.append(
                f"ln -sf {env['CM_ML_MODEL_FILE_WITH_PATH']} {model_path}")
        model_name = "dlrm"

    elif "dlrm-v2" in env['CM_MODEL']:
        model_name = "dlrm-v2"

    elif env['CM_MODEL'] == "retinanet":
        # print(env)
        dataset_path = env['CM_DATASET_OPENIMAGES_PATH']
        # return {'return': 1, 'error': 'error'}

        annotations_path = env['CM_DATASET_OPENIMAGES_ANNOTATIONS_DIR_PATH']
        target_data_path_dir = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'data', 'open-images-v6-mlperf')
        if not os.path.exists(target_data_path_dir):
            cmds.append(f"mkdir -p {target_data_path_dir}")
        target_data_path = os.path.join(target_data_path_dir, 'annotations')
        if not os.path.exists(target_data_path):
            cmds.append(f"ln -sf {annotations_path} {target_data_path}")

        target_data_path_dir = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'data',
            'open-images-v6-mlperf',
            'validation')
        if not os.path.exists(target_data_path_dir):
            cmds.append(f"mkdir -p {target_data_path_dir}")
        target_data_path = os.path.join(target_data_path_dir, 'data')
        if not os.path.exists(target_data_path):
            cmds.append(f"ln -sf {dataset_path} {target_data_path}")

        calibration_dataset_path = env['CM_OPENIMAGES_CALIBRATION_DATASET_PATH']
        target_data_path_dir = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'data',
            'open-images-v6-mlperf',
            'calibration',
            'train')
        if not os.path.exists(target_data_path_dir):
            cmds.append(f"mkdir -p {target_data_path_dir}")
        target_data_path = os.path.join(target_data_path_dir, 'data')
        if not os.path.exists(target_data_path):
            cmds.append(
                f"ln -sf {calibration_dataset_path} {target_data_path}")

        preprocessed_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'], 'preprocessed_data')
        target_model_path_dir = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'retinanet-resnext50-32x4d')
        if not os.path.exists(target_model_path_dir):
            cmds.append(f"mkdir -p {target_model_path_dir}")
        model_path = os.path.join(
            target_model_path_dir,
            'retinanet-fpn-torch2.1-postprocessed.onnx')
        alt_model_path = os.path.join(
            target_model_path_dir,
            'retinanet-fpn-torch2.2-postprocessed.onnx')
        if not os.path.exists(model_path) and os.path.exists(alt_model_path):
            cmds.append(f"ln -s {alt_model_path} {model_path}")

        model_name = "retinanet"

    elif "gptj" in env['CM_MODEL']:
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'data',
            'cnn-daily-mail',
            'cnn_eval.json')
        if not os.path.exists(target_data_path):
            cmds.append("make download_data BENCHMARKS='gptj'")

        fp32_model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'GPTJ-6B',
            'checkpoint-final')
        fp8_model_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'GPTJ-6B',
            'fp8-quantized-ammo',
            env['CM_MLPERF_GPTJ_MODEL_FP8_PATH_SUFFIX'])
        vocab_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'models',
            'bert',
            'vocab.txt')

        if not os.path.exists(os.path.dirname(fp32_model_path)):
            cmds.append(f"mkdir -p {os.path.dirname(fp32_model_path)}")
        if not os.path.exists(os.path.dirname(fp8_model_path)):
            cmds.append(f"mkdir -p {os.path.dirname(fp8_model_path)}")

        if not os.path.exists(fp32_model_path):
            # download via prehook_deps
            env['CM_REQUIRE_GPTJ_MODEL_DOWNLOAD'] = 'yes'
            if make_command == "build_engine":
                cmds.append(
                    f"cp -r $CM_ML_MODEL_FILE_WITH_PATH {fp32_model_path}")

        model_name = "gptj"
        model_path = fp8_model_path

    elif "llama2" in env["CM_MODEL"]:
        # path to which the data file is present
        target_data_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'preprocessed_data',
            'open_orca')
        # path to the dataset file
        target_data_file_path = os.path.join(
            env['MLPERF_SCRATCH_PATH'],
            'preprocessed_data',
            'open_orca',
            'open_orca_gpt4_tokenized_llama.sampled_24576.pkl')
        tmp_tp_size = env['CM_NVIDIA_TP_SIZE']
        if tmp_tp_size == "1":
            fp8_model_path = os.path.join(
                env['MLPERF_SCRATCH_PATH'],
                'models',
                'Llama2',
                'fp8-quantized-ammo',
                f'llama2-70b-chat-hf-tp{tmp_tp_size}pp1-fp8-02072024')
        else:
            fp8_model_path = os.path.join(
                env['MLPERF_SCRATCH_PATH'],
                'models',
                'Llama2',
                'fp8-quantized-ammo',
                f'llama2-70b-chat-hf-tp{tmp_tp_size}pp1-fp8')
        if not os.path.exists(target_data_file_path):
            if env.get('CM_NVIDIA_LLAMA_DATASET_FILE_PATH', '') == '':
                return {
                    'return': 1, 'error': 'Please specify the path to LLAMA2 dataset (pickle file)'}
            if not os.path.exists(target_data_path):
                cmds.append(f"mkdir {target_data_path}")
            cmds.append(
                f"ln -sf {env['CM_NVIDIA_LLAMA_DATASET_FILE_PATH']} {target_data_file_path}")

        model_name = "llama2-70b"
        model_path = fp8_model_path

    # cmds.append(f"make prebuild")
    if make_command == "download_model":
        if not os.path.exists(model_path):
            if "llama2" in env['CM_MODEL']:
                if not os.path.exists(os.path.join(model_path, 'config.json')):
                    return {
                        'return': 1, 'error': f'Quantised model absent - did not detect config.json in path {model_path}'}
            else:
                cmds.append(f"make download_model BENCHMARKS='{model_name}'")
        elif "stable-diffusion" in env['CM_MODEL']:
            folders = ["clip1", "clip2", "unetxl", "vae"]
            for folder in folders:
                onnx_model_path = os.path.join(
                    env['MLPERF_SCRATCH_PATH'],
                    'models',
                    'SDXL',
                    'onnx_models',
                    folder,
                    'model.onnx')
                if not os.path.exists(onnx_model_path):
                    env['CM_REQUIRE_SDXL_MODEL_DOWNLOAD'] = 'yes'
                    cmds.append(
                        f"make download_model BENCHMARKS='{model_name}'")
                    break
            if scenario.lower() == "singlestream":
                ammo_model_path = os.path.join(
                    env['MLPERF_SCRATCH_PATH'],
                    'models',
                    'SDXL',
                    'ammo_models',
                    'unetxl.int8',
                    'unet.onnx')
                if not os.path.exists(ammo_model_path):
                    env['CM_REQUIRE_SDXL_MODEL_DOWNLOAD'] = 'yes'
                    cmds.append(
                        f"make download_model BENCHMARKS='{model_name}'")
        else:
            return {'return': 0}

    elif make_command == "preprocess_data":
        if env['CM_MODEL'] == "rnnt":
            cmds.append(
                f"rm -rf {os.path.join(env['MLPERF_SCRATCH_PATH'], 'preprocessed_data', 'rnnt_dev_clean_500_raw')}")
            cmds.append(
                f"rm -rf {os.path.join(env['MLPERF_SCRATCH_PATH'], 'preprocessed_data', 'rnnt_train_clean_512_wav')}")
        if "llama2" in env["CM_MODEL"]:
            # Preprocessing script in the inference results repo is not checking whether the preprocessed
            # file is already there, so we are handling it here.
            target_preprocessed_data_path = os.path.join(
                env['MLPERF_SCRATCH_PATH'],
                'preprocessed_data',
                'open_orca',
                'input_ids_padded.npy')
            if not os.path.exists(target_preprocessed_data_path):
                cmds.append(f"make preprocess_data BENCHMARKS='{model_name}'")
        else:
            cmds.append(f"make preprocess_data BENCHMARKS='{model_name}'")

    else:
        scenario = scenario.lower()

        if env['CM_MLPERF_LOADGEN_MODE'] == "accuracy":
            test_mode = "AccuracyOnly"
        elif env['CM_MLPERF_LOADGEN_MODE'] == "performance":
            test_mode = "PerformanceOnly"
        elif env['CM_MLPERF_LOADGEN_MODE'] == "compliance":
            test_mode = ""
            test_name = env.get(
                'CM_MLPERF_LOADGEN_COMPLIANCE_TEST',
                'test01').lower()
            env['CM_MLPERF_NVIDIA_RUN_COMMAND'] = "run_audit_{}_once".format(
                test_name)
            make_command = "run_audit_{}_once".format(test_name)
        else:
            return {'return': 1, 'error': 'Unsupported mode: {}'.format(
                env['CM_MLPERF_LOADGEN_MODE'])}

        run_config = ''

        target_qps = env.get('CM_MLPERF_LOADGEN_TARGET_QPS')
        offline_target_qps = env.get('CM_MLPERF_LOADGEN_OFFLINE_TARGET_QPS')
        server_target_qps = env.get('CM_MLPERF_LOADGEN_SERVER_TARGET_QPS')
        if target_qps:
            target_qps = int(float(target_qps))
            if scenario == "offline" and not offline_target_qps:
                run_config += f" --offline_expected_qps={target_qps}"
            elif scenario == "server" and not server_target_qps:
                run_config += f" --server_target_qps={target_qps}"

        if offline_target_qps:
            offline_target_qps = int(float(offline_target_qps))
            run_config += f" --offline_expected_qps={offline_target_qps}"
        if server_target_qps:
            server_target_qps = int(float(server_target_qps))
            run_config += f" --server_target_qps={server_target_qps}"

        target_latency = env.get('CM_MLPERF_LOADGEN_TARGET_LATENCY')
        singlestream_target_latency = env.get(
            'CM_MLPERF_LOADGEN_SINGLESTREAM_TARGET_LATENCY')
        multistream_target_latency = env.get(
            'CM_MLPERF_LOADGEN_MULTISTREAM_TARGET_LATENCY')
        if target_latency:
            target_latency_ns = int(float(target_latency) * 1000000)
            if scenario == "singlestream" and not singlestream_target_latency:
                run_config += f" --single_stream_expected_latency_ns={target_latency_ns}"
            elif scenario == "multistream" and not multistream_target_latency:
                run_config += f" --multi_stream_expected_latency_ns={target_latency_ns}"

        if singlestream_target_latency:
            singlestream_target_latency_ns = int(
                float(singlestream_target_latency) * 1000000)
            run_config += f" --single_stream_expected_latency_ns={singlestream_target_latency_ns}"
        if multistream_target_latency:
            multistream_target_latency_ns = int(
                float(multistream_target_latency) * 1000000)
            run_config += f" --multi_stream_expected_latency_ns={multistream_target_latency_ns}"

        high_accuracy = "99.9" in env['CM_MODEL']

        config_ver_list = []

        use_lon = env.get('CM_MLPERF_NVIDIA_HARNESS_LON')
        if use_lon:
            config_ver_list.append("lon_node")
            # run_config += " --lon_node"

        maxq = env.get('CM_MLPERF_NVIDIA_HARNESS_MAXQ')
        if maxq:
            config_ver_list.append("maxq")

        if high_accuracy:
            config_ver_list.append("high_accuracy")

        use_triton = env.get('CM_MLPERF_NVIDIA_HARNESS_USE_TRITON')
        if use_triton:
            run_config += " --use_triton "
            config_ver_list.append("triton")

        if config_ver_list:
            run_config += f" --config_ver={'_'.join(config_ver_list)}"

        user_conf_path = env.get('CM_MLPERF_USER_CONF')
        if user_conf_path and env['CM_MLPERF_NVIDIA_HARNESS_RUN_MODE'] == "run_harness":
            run_config += f" --user_conf_path={user_conf_path}"

        mlperf_conf_path = env.get('CM_MLPERF_INFERENCE_CONF_PATH')
        if mlperf_conf_path and env['CM_MLPERF_NVIDIA_HARNESS_RUN_MODE'] == "run_harness":
            run_config += f" --mlperf_conf_path={mlperf_conf_path}"

        power_setting = env.get('CM_MLPERF_NVIDIA_HARNESS_POWER_SETTING')
        if power_setting and env['CM_MLPERF_NVIDIA_HARNESS_RUN_MODE'] == "run_harness":
            run_config += f" --power_setting={power_setting}"

        gpu_copy_streams = env.get('CM_MLPERF_NVIDIA_HARNESS_GPU_COPY_STREAMS')
        if gpu_copy_streams:
            run_config += f" --gpu_copy_streams={gpu_copy_streams}"

        gpu_inference_streams = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_GPU_INFERENCE_STREAMS')
        if gpu_inference_streams:
            run_config += f" --gpu_inference_streams={gpu_inference_streams}"

        dla_copy_streams = env.get('CM_MLPERF_NVIDIA_HARNESS_DLA_COPY_STREAMS')
        if dla_copy_streams:
            run_config += f" --dla_copy_streams={dla_copy_streams}"

        dla_inference_streams = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_DLA_INFERENCE_STREAMS')
        if dla_inference_streams:
            run_config += f" --dla_inference_streams={dla_inference_streams}"

        gpu_batch_size = env.get('CM_MLPERF_NVIDIA_HARNESS_GPU_BATCH_SIZE')
        if gpu_batch_size:
            run_config += f" --gpu_batch_size={gpu_batch_size}"

        dla_batch_size = env.get('CM_MLPERF_NVIDIA_HARNESS_DLA_BATCH_SIZE')
        if dla_batch_size:
            run_config += f" --dla_batch_size={dla_batch_size}"

        input_format = env.get('CM_MLPERF_NVIDIA_HARNESS_INPUT_FORMAT')
        if input_format:
            run_config += f" --input_format={input_format}"

        performance_sample_count = env.get(
            'CM_MLPERF_LOADGEN_PERFORMANCE_SAMPLE_COUNT')
        if performance_sample_count:
            run_config += f" --performance_sample_count={performance_sample_count}"

        devices = env.get('CM_MLPERF_NVIDIA_HARNESS_DEVICES')
        if devices:
            run_config += f" --devices={devices}"

        audio_batch_size = env.get('CM_MLPERF_NVIDIA_HARNESS_AUDIO_BATCH_SIZE')
        if audio_batch_size:
            run_config += f" --audio_batch_size={audio_batch_size}"

        disable_encoder_plugin = str(
            env.get('CM_MLPERF_NVIDIA_HARNESS_DISABLE_ENCODER_PLUGIN', ''))
        if disable_encoder_plugin and disable_encoder_plugin.lower() not in [
                "no", "false", "0", ""]:
            run_config += " --disable_encoder_plugin"

        disable_beta1_smallk = str(
            env.get('CM_MLPERF_NVIDIA_HARNESS_DISABLE_BETA1_SMALLK', ''))
        if disable_beta1_smallk and disable_beta1_smallk.lower() in [
                "yes", "true", "1"]:
            run_config += " --disable_beta1_smallk"

        workspace_size = env.get('CM_MLPERF_NVIDIA_HARNESS_WORKSPACE_SIZE')
        if workspace_size:
            run_config += f" --workspace_size={workspace_size}"

        if env.get('CM_MLPERF_LOADGEN_LOGS_DIR'):
            env['MLPERF_LOADGEN_LOGS_DIR'] = env['CM_MLPERF_LOADGEN_LOGS_DIR']

        log_dir = env.get('CM_MLPERF_NVIDIA_HARNESS_LOG_DIR')
        if log_dir:
            run_config += f" --log_dir={log_dir}"

        use_graphs = str(env.get('CM_MLPERF_NVIDIA_HARNESS_USE_GRAPHS', ''))
        if use_graphs and use_graphs.lower() not in ["no", "false", "0", ""]:
            run_config += " --use_graphs"

        use_deque_limit = str(
            env.get('CM_MLPERF_NVIDIA_HARNESS_USE_DEQUE_LIMIT'))
        if use_deque_limit and use_deque_limit.lower() not in [
                "no", "false", "0"]:
            run_config += " --use_deque_limit"

            deque_timeout_usec = env.get(
                'CM_MLPERF_NVIDIA_HARNESS_DEQUE_TIMEOUT_USEC')
            if deque_timeout_usec:
                run_config += f" --deque_timeout_usec={deque_timeout_usec}"

        use_cuda_thread_per_device = str(
            env.get('CM_MLPERF_NVIDIA_HARNESS_USE_CUDA_THREAD_PER_DEVICE', ''))
        if use_cuda_thread_per_device and use_cuda_thread_per_device.lower() not in [
                "no", "false", "0", ""]:
            run_config += " --use_cuda_thread_per_device"

        run_infer_on_copy_streams = str(
            env.get('CM_MLPERF_NVIDIA_HARNESS_RUN_INFER_ON_COPY_STREAMS', ''))
        if run_infer_on_copy_streams and run_infer_on_copy_streams.lower() not in [
                "no", "false", "0", ""]:
            run_config += " --run_infer_on_copy_streams"

        start_from_device = str(
            env.get(
                'CM_MLPERF_NVIDIA_HARNESS_START_FROM_DEVICE',
                ''))
        if start_from_device and start_from_device.lower() not in [
                "no", "false", "0", ""]:
            run_config += " --start_from_device"

        end_on_device = str(
            env.get(
                'CM_MLPERF_NVIDIA_HARNESS_END_ON_DEVICE',
                ''))
        if end_on_device and end_on_device.lower() not in [
                "no", "false", "0", ""]:
            run_config += " --end_on_device"

        max_dlas = env.get('CM_MLPERF_NVIDIA_HARNESS_MAX_DLAS')
        if max_dlas:
            run_config += f" --max_dlas={max_dlas}"

        graphs_max_seqlen = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_GRAPHS_MAX_SEQLEN')
        if graphs_max_seqlen:
            run_config += f" --graphs_max_seqlen={graphs_max_seqlen}"

        num_issue_query_threads = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_NUM_ISSUE_QUERY_THREADS')
        if num_issue_query_threads:
            run_config += f" --num_issue_query_threads={num_issue_query_threads}"

        soft_drop = env.get('CM_MLPERF_NVIDIA_HARNESS_SOFT_DROP')
        if soft_drop:
            run_config += f" --soft_drop={soft_drop}"

        use_small_tile_gemm_plugin = str(
            env.get('CM_MLPERF_NVIDIA_HARNESS_USE_SMALL_TILE_GEMM_PLUGIN', ''))
        if use_small_tile_gemm_plugin and use_small_tile_gemm_plugin.lower() not in [
                "no", "false", "0", ""]:
            run_config += f" --use_small_tile_gemm_plugin"

        audio_buffer_num_lines = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_AUDIO_BUFFER_NUM_LINES')
        if audio_buffer_num_lines:
            run_config += f" --audio_buffer_num_lines={audio_buffer_num_lines}"

        use_fp8 = str(env.get('CM_MLPERF_NVIDIA_HARNESS_USE_FP8', ''))
        if use_fp8 and use_fp8.lower() not in ["no", "false", "0", ""]:
            run_config += f" --use_fp8"

        if "llama2" in env["CM_MODEL"]:
            run_config += f" --fp8_quant_model_path={fp8_model_path}"
            run_config += f" --tensor_parallelism={tmp_tp_size}"

        enable_sort = env.get('CM_MLPERF_NVIDIA_HARNESS_ENABLE_SORT')
        if enable_sort and enable_sort.lower() not in ["no", "false", "0"]:
            run_config += f" --enable_sort"

        sdxl_server_batcher_time_limit = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_ENABLE_SORT')
        if sdxl_server_batcher_time_limit:
            run_config += f" --sdxl_batcher_time_limit {sdxl_server_batcher_time_limit}"

        num_sort_segments = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_NUM_SORT_SEGMENTS')
        if num_sort_segments:
            run_config += f" --num_sort_segments={num_sort_segments}"

        embedding_weights_on_gpu_part = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_EMBEDDING_WEIGHTS_ON_GPU_PART', '')
        if embedding_weights_on_gpu_part != '':
            run_config += f" --embedding_weights_on_gpu_part={embedding_weights_on_gpu_part}"

        num_warmups = env.get('CM_MLPERF_NVIDIA_HARNESS_NUM_WARMUPS', '')
        if num_warmups != '':
            run_config += f" --num_warmups={num_warmups}"

        skip_postprocess = str(
            env.get(
                'CM_MLPERF_NVIDIA_HARNESS_SKIP_POSTPROCESS',
                ''))
        if skip_postprocess and skip_postprocess.lower() not in [
                "no", "false", "0", ""]:
            run_config += f" --skip_postprocess"

        if test_mode:
            test_mode_string = " --test_mode={}".format(test_mode)
        else:
            test_mode_string = ""

        extra_build_engine_options_string = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_EXTRA_BUILD_ENGINE_OPTIONS', '')

        extra_run_options_string = env.get(
            'CM_MLPERF_NVIDIA_HARNESS_EXTRA_RUN_OPTIONS',
            '')  # will be ignored during build engine

        if "stable-diffusion" in env["CM_MODEL"]:
            extra_build_engine_options_string += f""" --model_path {
                os.path.join(
                    env['MLPERF_SCRATCH_PATH'],
                    'models',
                    'SDXL/')}"""

        run_config += " --no_audit_verify"

        cmds.append(f"""make {make_command} RUN_ARGS=' --benchmarks={model_name} --scenarios={scenario} {test_mode_string} {run_config} {extra_build_engine_options_string} {extra_run_options_string}'""")

    run_cmd = " && ".join(cmds)
    env['CM_MLPERF_RUN_CMD'] = run_cmd
    env['CM_RUN_CMD'] = run_cmd
    env['CM_RUN_DIR'] = env['CM_MLPERF_INFERENCE_NVIDIA_CODE_PATH']

#    print(env)

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    return {'return': 0}



from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



import pandas as pd
import argparse
import os


def main():
    # Set up argument parser
    parser = argparse.ArgumentParser(
        description="Sample test dataset from the original dataset.")
    parser.add_argument(
        '--dataset-path',
        required=True,
        help="Path to the input dataset (pickle file).")
    parser.add_argument(
        '--output-path',
        default=os.path.join(
            os.getcwd(),
            "mixtral-test-dataset.pkl"),
        help="Path to save the output dataset (pickle file).")
    parser.add_argument(
        '--samples',
        default=2,
        help="Number of entries to be extracted from each group.")

    args = parser.parse_args()
    dataset_path = args.dataset_path
    output_path = args.output_path
    no_of_samples = int(args.samples)

    try:
        # Load the dataset from the specified pickle file
        print(f"Loading dataset from {dataset_path}...")
        df = pd.read_pickle(dataset_path)

        # Check if 'group' column exists
        if 'dataset' not in df.columns:
            raise ValueError(
                "The input dataset must contain a 'dataset' column to identify data set groups.")

        # Sample 2 entries from each group
        print(f"Sampling {no_of_samples} entries from each group...")
        sampled_df = df.groupby('dataset').apply(
            lambda x: x.sample(
                n=no_of_samples)).reset_index(
            drop=True)

        # Save the sampled dataset to the specified output path
        print(f"Saving the sampled dataset to {output_path}...")
        sampled_df.to_pickle(output_path)

        print("Dataset processing and saving completed successfully!")
    except Exception as e:
        print(f"Error: {e}")
        exit(1)


if __name__ == '__main__':
    main()



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    env['CM_BLIS_SRC_PATH'] = env['CM_GIT_CHECKOUT_PATH']

    return {'return': 0}


def postprocess(i):

    env = i['env']
    install_dir = os.path.join(env['CM_BLIS_SRC_PATH'], "install")

    env['CM_BLIS_INSTALL_PATH'] = install_dir
    env['CM_BLIS_INC'] = os.path.join(install_dir, 'include', 'blis')
    env['CM_BLIS_LIB'] = os.path.join(install_dir, 'lib', 'libblis.a')

    blis_lib_path = os.path.join(install_dir, 'lib')

    env['+LD_LIBRARY_PATH'] = [blis_lib_path] if '+LD_LIBRARY_PATH' not in env else env['+LD_LIBRARY_PATH'] + [blis_lib_path]

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

# Developer(s): Grigori Fursin

import os


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    meta = i['meta']

    print("********************************************************")
    print('- Importing CM library ...')
    import cmind
    print('  SUCCESS!')

    cmind.utils.debug_here(
        __file__,
        port=5678,
        text='Debugging customize.py!',
        env=env,
        env_debug_uid='8d96cd9fa4734204').breakpoint()

    print('')
    print('- List CM repos ...')
    print('')
    r = cmind.access({'action': 'show', 'automation': 'repo', 'out': 'con'})
    print('')
    print('  SUCCESS!')
    print("********************************************************")

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    return {'return': 0}



import abc
import contextlib
import logging
import typing

import mlperf_loadgen

from loadgen.model import ModelInput, ModelInputSampler

logger = logging.getLogger(__name__)


QueryInput = typing.Dict[int, ModelInput]
QueryResult = typing.Dict[int, typing.Any]


class ModelRunner(contextlib.AbstractContextManager):
    @abc.abstractmethod
    def issue_query(self, query: QueryInput) -> typing.Optional[QueryResult]:
        pass

    # Optional method to flush pending queries
    def flush_queries(self) -> typing.Optional[QueryResult]:
        pass

    def __exit__(self, _exc_type, _exc_value, _traceback):
        logger.info(f"{self} : Exited")
        return None


class Harness:
    def __init__(self, sampler: ModelInputSampler, runner: ModelRunner):
        self.sampler = sampler
        self.runner = runner
        self.samples = None

    def load_query_samples(self, query_samples):
        assert self.samples is None
        self.samples = dict()
        for query_id in query_samples:
            self.samples[query_id] = self.sampler.sample(query_id)
        logger.info(f"Loaded {len(self.samples)} samples")

    def unload_query_samples(self, _query_samples):
        assert self.samples is not None
        logger.info(f"Unloaded samples")
        self.samples = None

    def issue_query(self, query_samples):
        query_input = dict()
        for q in query_samples:
            # logger.info(f"Query Id: {q.id}, SampleIndex: {q.index}")
            input = self.samples[q.index]
            query_input[q.id] = input
        result = self.runner.issue_query(query_input)
        logger.info(f"Queries issued {len(query_input)}")
        if result is not None:
            self._complete_query(result)

    # Called after the last call to issue queries in a series is made.
    # Client can use this to flush any deferred queries rather than waiting
    # for a timeout.
    def flush_queries(self):
        result = self.runner.flush_queries()
        logger.info(f"Queries flushed")
        if result is not None:
            self._complete_query(result)

    def _complete_query(self, result: QueryResult):
        responses = []
        for query_id, _query_result in result.items():
            response_data, response_size = 0, 0
            response = mlperf_loadgen.QuerySampleResponse(
                query_id, response_data, response_size
            )
            responses.append(response)
        mlperf_loadgen.QuerySamplesComplete(responses)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    if '+PYTHONPATH' not in env:
        env['+PYTHONPATH'] = []
    env['+PYTHONPATH'].append(
        os.path.join(
            env['CM_MLPERF_TRAINING_SOURCE'],
            "single_stage_detector",
            "ssd"))
    env['CM_ML_MODEL_DYN_BATCHSIZE_PATH'] = os.path.join(
        os.getcwd(), "retinanet_resnext50_32x4d_fpn.opset11.dyn_bs.800x800.onnx")
    if "CM_NVIDIA_EFFICIENT_NMS" in env:
        env['CM_NVIDIA_MODEL_PATCHED_PATH'] = os.path.join(
            os.getcwd(), "fpn_efficientnms_concatall.onnx")
        env['CM_ML_MODEL_ANCHOR_PATH'] = os.path.join(
            env['CM_MLPERF_INFERENCE_NVIDIA_CODE_PATH'],
            "code",
            "retinanet",
            "tensorrt",
            "onnx_generator",
            "retinanet_anchor_xywh_1x1.npy")
    return {'return': 0}


def postprocess(i):
    env = i['env']
    env['CM_NVIDIA_RETINANET_EFFICIENT_NMS_CONCAT_MODEL_WITH_PATH'] = os.path.join(
        os.getcwd(), "test_fpn_efficientnms_concatall.onnx")
    if "CM_NVIDIA_EFFICIENT_NMS" in env:
        env['CM_NVIDIA_RETINANET_EFFICIENT_NMS_CONCAT_MODEL_WITH_PATH'] = env['CM_NVIDIA_MODEL_PATCHED_PATH']
    return {'return': 0}



# Developer: Grigori Fursin

import os
import pandas


def main():
    print('=========================================================')

    print('Searching for summary.csv ...')

    if os.path.isfile('summary.csv'):
        print('Converting to json ...')

        import pandas

        df = pandas.read_csv('summary.csv').T

        print('')
        print(df)
        print('')

        df.to_json('summary.json', orient='columns', indent=4)

    print('=========================================================')


if __name__ == '__main__':
    main()



# Author: Grigori Fursin
# Contributors: Arjun Suresh, Anandhu Sooraj
#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import os

from cmind.automation import Automation
from cmind import utils


class CAutomation(Automation):
    """
    Automation actions
    """

    ############################################################
    def __init__(self, cmind, automation_file):
        super().__init__(cmind, __file__)

    ############################################################
    def test(self, i):
        """
        Test automation

        Args:
          (CM input dict):

          (out) (str): if 'con', output to console

          automation (str): automation as CM string object

          parsed_automation (list): prepared in CM CLI or CM access function
                                    [ (automation alias, automation UID) ] or
                                    [ (automation alias, automation UID), (automation repo alias, automation repo UID) ]

          (artifact) (str): artifact as CM string object

          (parsed_artifact) (list): prepared in CM CLI or CM access function
                                    [ (artifact alias, artifact UID) ] or
                                    [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action

        """

        import json
        print(json.dumps(i, indent=2))

        return {'return': 0}

    ############################################################
    def show(self, i):
        """
        Show cache

        Args:
          (CM input dict):

          (out) (str): if 'con', output to console

          (env) (bool): if True, show env from cm-cached-state.json
          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action

        """
        import json

        # Check parsed automation
        if 'parsed_automation' not in i:
            return {'return': 1, 'error': 'automation is not specified'}

        console = i.get('out') == 'con'

        show_env = i.get('env', False)

# Moved to search function
#        # Check simplified CMD: cm show cache "get python"
#        # If artifact has spaces, treat them as tags!
#        artifact = i.get('artifact','')
#        tags = i.get('tags','').strip()
#        if ' ' in artifact or ',' in artifact:
#            del(i['artifact'])
#            if 'parsed_artifact' in i: del(i['parsed_artifact'])
#
#            new_tags = artifact.replace(' ',',')
#            tags = new_tags if tags=='' else new_tags+','+tags
#
#            i['tags'] = tags

        # Find CM artifact(s)
        i['out'] = None
        r = self.search(i)

        if r['return'] > 0:
            return r

        lst = r['list']
        for artifact in sorted(lst, key=lambda x: sorted(x.meta['tags'])):
            #        for artifact in lst:
            path = artifact.path
            meta = artifact.meta
            dependent_cached_path = meta.get(
                'dependent_cached_path', '')
            if dependent_cached_path and not os.path.exists(
                    dependent_cached_path):
                continue

            original_meta = artifact.original_meta

            alias = meta.get('alias', '')
            uid = meta.get('uid', '')

            tags = meta.get('tags', [])
            tags1 = sorted([x for x in tags if not x.startswith('_')])
            tags2 = sorted([x for x in tags if x.startswith('_')])
            tags = tags1 + tags2

            version = meta.get('version', '')

            if console:
                print('')
#                print ('* UID: {}'.format(uid))
                print('* Tags: {}'.format(','.join(tags)))
                print('  Path: {}'.format(path))
                if version != '':
                    print('  Version: {}'.format(version))

            if show_env and console:
                path_to_cached_state_file = os.path.join(
                    path, 'cm-cached-state.json')

                if os.path.isfile(path_to_cached_state_file):
                    r = utils.load_json(file_name=path_to_cached_state_file)
                    if r['return'] > 0:
                        return r

                    # Update env and state from cache!
                    cached_state = r['meta']

                    new_env = cached_state.get('new_env', {})
                    if len(new_env) > 0:
                        print('    New env:')
                        print(
                            json.dumps(
                                new_env,
                                indent=6,
                                sort_keys=True).replace(
                                '{',
                                '').replace(
                                '}',
                                ''))

                    new_state = cached_state.get('new_state', {})
                    if len(new_state) > 0:
                        print('    New state:')
                        print(json.dumps(new_env, indent=6, sort_keys=True))

        return {'return': 0, 'list': lst}

    ############################################################
    def search(self, i):
        """
        Overriding the automation search function to add support for a simplified CMD with tags with spaces

        TBD: add input/output description
        """
        # Check simplified CMD: cm show cache "get python"
        # If artifact has spaces, treat them as tags!
        artifact = i.get('artifact', '')
        tags = i.get('tags', '')

        # Tags may be a list (if comes internally from CM scripts) or string if
        # comes from CMD
        if not isinstance(tags, list):
            tags = tags.strip()

        if ' ' in artifact:  # or ',' in artifact:
            del (i['artifact'])
            if 'parsed_artifact' in i:
                del (i['parsed_artifact'])

            new_tags = artifact.replace(' ', ',')
            tags = new_tags if tags == '' else new_tags + ',' + tags

            i['tags'] = tags

        # Force automation when reruning access with processed input
        i['automation'] = 'cache,541d6f712a6b464e'
        i['action'] = 'search'
        # Avoid recursion - use internal CM add function to add the script
        # artifact
        i['common'] = True

        # Find CM artifact(s)
        return self.cmind.access(i)

    ############################################################

    def copy_to_remote(self, i):
        """
        Add CM automation.

        Args:
          (CM input dict):

          (out) (str): if 'con', output to console

          parsed_artifact (list): prepared in CM CLI or CM access function
                                    [ (artifact alias, artifact UID) ] or
                                    [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]

          (repos) (str): list of repositories to search for automations (internal & mlcommons@ck by default)

          (output_dir) (str): output directory (./ by default)

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

        """

        return utils.call_internal_module(
            self, __file__, 'module_misc', 'copy_to_remote', i)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import subprocess


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    q = '"' if os_info['platform'] == 'windows' else "'"

    submission_dir = env.get("CM_MLPERF_INFERENCE_SUBMISSION_DIR", "")

    version = env.get('CM_MLPERF_SUBMISSION_CHECKER_VERSION', '')

    if submission_dir == "":
        return {'return': 1,
                'error': 'Please set --env.CM_MLPERF_INFERENCE_SUBMISSION_DIR'}

    submitter = env.get("CM_MLPERF_SUBMITTER", "")  # "default")
    if ' ' in submitter:
        return {
            'return': 1, 'error': 'CM_MLPERF_SUBMITTER cannot contain a space. Please provide a name without space using --submitter input. Given value: {}'.format(submitter)}

    if 'CM_MLPERF_SKIP_COMPLIANCE' in env:
        skip_compliance = " --skip_compliance"
    else:
        skip_compliance = ""

    submission_checker_file = os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "tools", "submission",
                                           "submission_checker.py")

    if env['CM_MLPERF_SHORT_RUN'] == "yes":
        import shutil
        new_submission_checker_file = os.path.join(
            os.path.dirname(submission_checker_file),
            "submission_checker1.py")
        with open(submission_checker_file, 'r') as file:
            data = file.read()
        data = data.replace("OFFLINE_MIN_SPQ = 24576", "OFFLINE_MIN_SPQ = 100")
        data = data.replace(
            "return is_valid, res, inferred",
            "return True, res, inferred")
        with open(new_submission_checker_file, 'w') as file:
            file.write(data)
        submission_checker_file = new_submission_checker_file

    if env.get('CM_MLPERF_EXTRA_MODEL_MAPPING', '') != '':
        extra_map = ' --extra_model_benchmark_map "' + \
            env['CM_MLPERF_EXTRA_MODEL_MAPPING'] + '"'
    else:
        extra_map = ""

    if env.get('CM_MLPERF_SKIP_POWER_CHECK', 'no') == "yes":
        power_check = " --skip-power-check"
    else:
        power_check = ""

    extra_args = ' ' + env.get('CM_MLPERF_SUBMISSION_CHECKER_EXTRA_ARGS', '')

    x_submitter = ' --submitter ' + q + submitter + q if submitter != '' else ''

    x_version = ' --version ' + version + ' ' if version != '' else ''

    CMD = env['CM_PYTHON_BIN_WITH_PATH'] + ' ' + q + submission_checker_file + q + ' --input ' + q + submission_dir + q + \
        x_submitter + \
        x_version + \
        skip_compliance + extra_map + power_check + extra_args

    x_version = ' --version ' + version[1:] + ' ' if version != '' else ''

    x_submission_repo_name = ''
    x_submission_repo_owner = ''
    x_submission_repo_branch = ''

    if env.get('CM_MLPERF_RESULTS_GIT_REPO_NAME', '') != '':
        x_submission_repo_name = f""" --repository {env['CM_MLPERF_RESULTS_GIT_REPO_NAME']}"""
    if env.get('CM_MLPERF_RESULTS_GIT_REPO_OWNER', '') != '':
        x_submission_repo_owner = f""" --repository-owner {env['CM_MLPERF_RESULTS_GIT_REPO_OWNER']}"""
    if env.get('CM_MLPERF_RESULTS_GIT_REPO_BRANCH', '') != '':
        x_submission_repo_branch = f""" --repository-branch  {env['CM_MLPERF_RESULTS_GIT_REPO_BRANCH']}"""

    report_generator_file = os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "tools", "submission",
                                         "generate_final_report.py")
    env['CM_RUN_CMD'] = CMD
    print(CMD)
    env['CM_POST_RUN_CMD'] = env['CM_PYTHON_BIN_WITH_PATH'] + ' ' + q + report_generator_file + q + ' --input summary.csv ' + \
        x_version + \
        x_submission_repo_name + \
        x_submission_repo_owner + \
        x_submission_repo_branch

    return {'return': 0}


def postprocess(i):

    env = i['env']
    if env.get('CM_TAR_SUBMISSION_DIR', ''):
        env['CM_TAR_INPUT_DIR'] = env['CM_MLPERF_INFERENCE_SUBMISSION_DIR']

    x = env.get('MLPERF_INFERENCE_SUBMISSION_TAR_FILE', '')
    if x != '':
        env['CM_TAR_OUTFILE'] = x

    if env.get('CM_MLPERF_INFERENCE_SUBMISSION_BASE_DIR', '') != '':
        env['CM_TAR_OUTPUT_DIR'] = env['CM_MLPERF_INFERENCE_SUBMISSION_BASE_DIR']

    x = env.get('MLPERF_INFERENCE_SUBMISSION_SUMMARY', '')
    if x != '':
        for y in ['.csv', '.json', '.xlsx']:

            z0 = 'summary' + y

            if os.path.isfile(z0):
                z1 = x + y

                if os.path.isfile(z1):
                    os.remove(z1)

                os.rename(z0, z1)

    return {'return': 0}



import argparse
import contextlib
import logging
import os
import re
import typing

import mlperf_loadgen
import psutil

from loadgen.harness import Harness, ModelRunner
from loadgen.runners import (
    ModelRunnerInline,
    ModelRunnerMultiProcessingPool,
    ModelRunnerProcessPoolExecutor,
    ModelRunnerThreadPoolExecutor,
    ModelRunnerThreadPoolExecutorWithTLS,
)

logger = logging.getLogger(__name__)


def main(
    backend: str,
    model_path: str,
    model_code: str,
    model_cfg: str,
    model_sample_pickle: str,
    output_path: typing.Optional[str],
    runner_name: str,
    runner_concurrency: int,
    execution_provider: str,
    execution_mode: str,
    intraop_threads: int,
    interop_threads: int,
    samples: int,
    loadgen_expected_qps: float,
    loadgen_duration_sec: float
):

    print('=====================================================================')

    if backend == 'onnxruntime':
        from backend_onnxruntime import XModelFactory
        from backend_onnxruntime import XModelInputSampler
    elif backend == 'pytorch':
        from backend_pytorch import XModelFactory
        from backend_pytorch import XModelInputSampler
    else:
        raise Exception("Error: backend is not recognized.")

    # Load model cfg
    model_cfg_dict = {}
    if model_cfg != '':
        import json

        with open(model_cfg) as mc:
            model_cfg_dict = json.load(mc)

    model_factory = XModelFactory(
        model_path,
        execution_provider,
        execution_mode,
        interop_threads,
        intraop_threads,
        model_code,
        model_cfg_dict,
        model_sample_pickle
    )

    model_dataset = XModelInputSampler(model_factory)

    runner: ModelRunner = None
    if runner_name == "inline":
        runner = ModelRunnerInline(model_factory)
    elif runner_name == "threadpool":
        runner = ModelRunnerThreadPoolExecutor(
            model_factory, max_concurrency=runner_concurrency
        )
    elif runner_name == "threadpool+replication":
        runner = ModelRunnerThreadPoolExecutorWithTLS(
            model_factory, max_concurrency=runner_concurrency
        )
    elif runner_name == "processpool":
        runner = ModelRunnerProcessPoolExecutor(
            model_factory, max_concurrency=runner_concurrency
        )
    elif runner_name == "processpool+mp":
        runner = ModelRunnerMultiProcessingPool(
            model_factory, max_concurrency=runner_concurrency
        )
    else:
        raise ValueError(f"Invalid runner {runner}")

    settings = mlperf_loadgen.TestSettings()

    settings.scenario = mlperf_loadgen.TestScenario.Offline
    settings.mode = mlperf_loadgen.TestMode.PerformanceOnly
    settings.offline_expected_qps = loadgen_expected_qps
    settings.min_query_count = samples
    settings.max_query_count = samples
    settings.min_duration_ms = loadgen_duration_sec * 1000
    # Duration isn't enforced in offline mode
    # Instead, it is used to determine total sample count via
    # target_sample_count = Slack (1.1) * TargetQPS (1) * TargetDuration ()
    # samples_per_query = Max(min_query_count, target_sample_count)

    output_path = "results" if not output_path else output_path
    output_path = os.path.join(
        output_path,
        os.path.basename(model_path),
        runner_name)
    os.makedirs(output_path, exist_ok=True)

    output_settings = mlperf_loadgen.LogOutputSettings()
    output_settings.outdir = output_path
    output_settings.copy_summary_to_stdout = True

    log_settings = mlperf_loadgen.LogSettings()
    log_settings.log_output = output_settings
    log_settings.enable_trace = False

    logger.info(f"Model: {model_path}")
    logger.info(f"Runner: {runner_name}, Concurrency: {runner_concurrency}")
    logger.info(f"Results: {output_path}")

    with contextlib.ExitStack() as stack:
        stack.enter_context(runner)
        harness = Harness(model_dataset, runner)

        query_sample_libary = mlperf_loadgen.ConstructQSL(
            samples,  # Total sample count
            samples,  # Num to load in RAM at a time
            harness.load_query_samples,
            harness.unload_query_samples,
        )
        system_under_test = mlperf_loadgen.ConstructSUT(
            harness.issue_query, harness.flush_queries
        )

        print('=====================================================================')
        logger.info("Test Started")

        mlperf_loadgen.StartTestWithLogSettings(
            system_under_test, query_sample_libary, settings, log_settings
        )

        logger.info("Test Finished")
        print('=====================================================================')

        # Parse output file
        output_summary = {}
        output_summary_path = os.path.join(
            output_path, "mlperf_log_summary.txt")
        with open(output_summary_path, "r") as output_summary_file:
            for line in output_summary_file:
                m = re.match(
                    r"^\s*([\w\s.\(\)\/]+)\s*\:\s*([\w\+\.]+).*", line)
                if m:
                    output_summary[m.group(1).strip()] = m.group(2).strip()
        logger.info(
            "Observed QPS: " +
            output_summary.get("Samples per second"))
        logger.info("Result: " + output_summary.get("Result is"))

        mlperf_loadgen.DestroySUT(system_under_test)
        mlperf_loadgen.DestroyQSL(query_sample_libary)
        logger.info("Test Completed")
        print('=====================================================================')


if __name__ == "__main__":
    print('')

    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s %(levelname)s %(threadName)s - %(name)s %(funcName)s: %(message)s",
    )

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "model_path", help="path to input model", default="models/yolov5s.onnx"
    )
    parser.add_argument(
        "-b",
        "--backend",
        help="backend",
        default="onnxruntime")
    parser.add_argument("-o", "--output", help="path to store loadgen results")
    parser.add_argument(
        "-r",
        "--runner",
        help="model runner",
        choices=[
            "inline",
            "threadpool",
            "threadpool+replication",
            "processpool",
            "processpool+mp",
        ],
        default="inline",
    )
    parser.add_argument(
        "--concurrency",
        help="concurrency count for runner",
        default=psutil.cpu_count(False),
        type=int,
    )
    parser.add_argument(
        "--ep", help="Execution Provider", default="CPUExecutionProvider"
    )
    parser.add_argument(
        "--intraop",
        help="IntraOp threads",
        default=0,
        type=int)
    parser.add_argument(
        "--interop",
        help="InterOp threads",
        default=0,
        type=int)
    parser.add_argument(
        "--execmode",
        help="Execution Mode",
        choices=["sequential", "parallel"],
        default="sequential",
    )
    parser.add_argument(
        "--samples",
        help="number of samples",
        default=100,
        type=int,
    )
    parser.add_argument(
        "--loadgen_expected_qps",
        help="Expected QPS",
        default=1,
        type=float)
    parser.add_argument(
        "--loadgen_duration_sec",
        help="Expected duration in sec.",
        default=1,
        type=float)
    parser.add_argument(
        "--model_code",
        help="(for PyTorch models) path to model code with cmc.py",
        default="")
    parser.add_argument(
        "--model_cfg",
        help="(for PyTorch models) path to model's configuration in JSON file",
        default="")
    parser.add_argument(
        "--model_sample_pickle",
        help="(for PyTorch models) path to a model sample in pickle format",
        default="")

    args = parser.parse_args()
    main(
        args.backend,
        args.model_path,
        args.model_code,
        args.model_cfg,
        args.model_sample_pickle,
        args.output,
        args.runner,
        args.concurrency,
        args.ep,
        args.execmode,
        args.intraop,
        args.interop,
        args.samples,
        args.loadgen_expected_qps,
        args.loadgen_duration_sec
    )



from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    # skip download and register in cache if the llama3 checkpoint path is
    # already defined by the user
    if env.get('CM_ML_MODEL_LLAMA3_CHECKPOINT_PATH', '') != '':
        env['LLAMA3_CHECKPOINT_PATH'] = env['CM_ML_MODEL_LLAMA3_CHECKPOINT_PATH']
        return {'return': 0}

    path = env.get('CM_OUTDIRNAME', '').strip()

    if path != "":
        os.makedirs(path, exist_ok=True)
        env['CM_GIT_CHECKOUT_FOLDER'] = os.path.join(
            path, env['CM_ML_MODEL_NAME'])

    env['CM_TMP_REQUIRE_DOWNLOAD'] = 'yes'

    return {'return': 0}


def postprocess(i):

    env = i['env']

    env['CM_ML_MODEL_LLAMA3_CHECKPOINT_PATH'] = env['LLAMA3_CHECKPOINT_PATH']
    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    if 'CM_MODEL' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the model to run'}
    if 'CM_MLPERF_DEVICE' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the device to run on'}

    # will later extend to other scenarios
    scenarios = env['CM_LOADGEN_SCENARIO']
    cmd = " --action generate_engines " +\
          " --benchmarks " + env['CM_MODEL'] + \
          " --scenarios " + scenarios + \
          " --gpu_batch_size=" + env['CM_MODEL_BATCH_SIZE'] +\
          " --gpu_copy_streams=" + env['CM_GPU_COPY_STREAMS'] +\
          " --workspace_size=" + env['CM_TENSORRT_WORKSPACE_SIZE']


~
return {'return': 0}


def postprocess(i):

    env = i['env']
    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

#    if os_info['platform'] == 'windows':
# return {'return':1, 'error': 'Windows is not supported in this script
# yet'}

    env = i['env']
    meta = i['meta']

    if 'CM_GIT_DEPTH' not in env:
        env['CM_GIT_DEPTH'] = ''

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    env['CM_MLPERF_TINY_SRC'] = os.path.join(os.getcwd(), 'src')
    env['CM_MLPERF_TINY_BENCHMARK'] = os.path.join(
        os.getcwd(), 'src', 'benchmark')
    env['CM_MLPERF_TINY_DATASETS'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'evaluation', 'datasets')
    env['CM_MLPERF_TINY_DATASETS_AD'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'evaluation', 'datasets', 'ad01')
    env['CM_MLPERF_TINY_DATASETS_IC'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'evaluation', 'datasets', 'ic01')
    env['CM_MLPERF_TINY_DATASETS_KWS'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'evaluation', 'datasets', 'kws01')
    env['CM_MLPERF_TINY_DATASETS_KWS_OPEN'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'evaluation', 'datasets', 'kws01-open')
    env['CM_MLPERF_TINY_DATASETS_VWW'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'evaluation', 'datasets', 'vww01')
    env['CM_MLPERF_TINY_TRAINING'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'training')
    env['CM_MLPERF_TINY_TRAINING_AD'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'training', 'anomaly_detection')
    env['CM_MLPERF_TINY_TRAINING_IC'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'training', 'image_classification')
    env['CM_MLPERF_TINY_TRAINING_KWS'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'training', 'keyword_spotting')
    env['CM_MLPERF_TINY_TRAINING_VWW'] = os.path.join(
        os.getcwd(), 'src', 'benchmark', 'training', 'visual_wake_words')

#        20221024: we save and restore env in the main script and can clean env here for determinism
#    if '+PYTHONPATH' not in env: env['+PYTHONPATH'] = []
#    env['+PYTHONPATH']=[]
#    env['+PYTHONPATH'].append(os.path.join(env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'], 'python'))
#    env['+PYTHONPATH'].append(os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], 'tools', 'submission'))

    return {'return': 0}



#!/usr/bin/env python3

import numpy as np
import cv2
import os
supported_extensions = ['jpeg', 'jpg', 'gif', 'png']


# Load and preprocess image

def load_image(image_path,            # Full path to processing image
               target_size,           # Desired size of resulting image
               intermediate_size=0,  # Scale to this size then crop to target size
               crop_percentage=87.5,  # Crop to this percentage then scale to target size
               data_type='uint8',   # Data type to store
               data_layout='nhwc',  # Data layout to store
               convert_to_bgr=False,  # Swap image channel RGB -> BGR
               interpolation_method=cv2.INTER_LINEAR  # Interpolation method.
               ):

    out_height = target_size
    out_width = target_size

    def resize_with_aspectratio(img):
        height, width, _ = img.shape
        # intermediate oversized image from which to crop
        new_height = int(100. * out_height / crop_percentage)
        # ---------------------- ,, ---------------------
        new_width = int(100. * out_width / crop_percentage)
        if height > width:
            w = new_width
            h = int(new_height * height / width)
        else:
            h = new_height
            w = int(new_width * width / height)
        img = cv2.resize(img, (w, h), interpolation=interpolation_method)
        return img

    def center_crop(img):
        height, width, _ = img.shape
        left = int((width - out_width) / 2)
        right = int((width + out_width) / 2)
        top = int((height - out_height) / 2)
        bottom = int((height + out_height) / 2)
        img = img[top:bottom, left:right]
        return img

    img = cv2.imread(image_path)

    if len(img.shape) < 3 or img.shape[2] != 3:
        img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGB)
    else:
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)

    # Mimic preprocessing steps from the official reference code.
    img = resize_with_aspectratio(img)
    img = center_crop(img)

    # Convert to BGR.
    if convert_to_bgr:
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)

    return img


def preprocess_files(selected_filenames, source_dir, destination_dir, crop_percentage, square_side, inter_size, convert_to_bgr,
                     data_type, data_layout, new_file_extension, normalize_data, subtract_mean, given_channel_means, given_channel_stds, quantize, quant_scale, quant_offset, convert_to_unsigned, interpolation_method):
    "Go through the selected_filenames and preprocess all the files (optionally normalize and subtract mean)"

    output_filenames = []

    for current_idx in range(len(selected_filenames)):
        input_filename = selected_filenames[current_idx]

        full_input_path = os.path.join(source_dir, input_filename)

        image_data = load_image(image_path=full_input_path,
                                target_size=square_side,
                                intermediate_size=inter_size,
                                crop_percentage=crop_percentage,
                                data_type=data_type,
                                convert_to_bgr=convert_to_bgr,
                                interpolation_method=interpolation_method)

        image_data = np.asarray(image_data, dtype=data_type)

        # Normalize.
        if normalize_data:
            image_data = image_data / 127.5 - 1.0

        # Subtract mean value.
        if subtract_mean:
            if len(given_channel_means):
                image_data -= given_channel_means
            else:
                image_data -= np.mean(image_data)

        # Subtract standard deviations.
        if len(given_channel_stds):
            image_data /= given_channel_stds

        # NHWC -> NCHW.
        if data_layout == 'nchw':
            image_data = image_data[:, :, 0:3].transpose(2, 0, 1)

        # Value 1 for quantization to int8
        if quantize == 1:
            image_data = quantize_to_int8(
                image_data, quant_scale, quant_offset)

        # Value 1 to convert from int8 to uint8
        if convert_to_unsigned == 1:
            image_data = int8_to_uint8(image_data)

        output_filename = input_filename.rsplit(
            '.', 1)[0] + '.' + new_file_extension if new_file_extension else input_filename

        full_output_path = os.path.join(destination_dir, output_filename)
        image_data.tofile(full_output_path)

        print("[{}]:  Stored {}".format(current_idx + 1, full_output_path))

        output_filenames.append(output_filename)

    return output_filenames


def quantize_to_int8(image, scale, offset):
    quant_image = (image / scale + offset).astype(np.float32)
    output = np.copy(quant_image)
    gtZero = (quant_image > 0).astype(int)
    gtZero = gtZero * 0.5
    output = output + gtZero
    ltZero = (quant_image < 0).astype(int)
    ltZero = ltZero * (-0.5)
    output = output + ltZero
    return output.astype(np.int8)


def int8_to_uint8(image):
    image = (image + 128).astype(np.uint8)
    return image


def preprocess():
    import sys

    source_dir = os.environ['CM_DATASET_PATH']
    destination_dir = os.environ['CM_DATASET_PREPROCESSED_PATH']

    square_side = int(os.environ['CM_DATASET_INPUT_SQUARE_SIDE'])
    crop_percentage = float(os.environ['CM_DATASET_CROP_FACTOR'])
    inter_size = int(os.getenv('CM_DATASET_INTERMEDIATE_SIZE', 0))
    convert_to_bgr = int(os.getenv('CM_DATASET_CONVERT_TO_BGR', 0))
    offset = int(os.getenv('CM_DATASET_SUBSET_OFFSET', 0))
    volume = int(os.environ['CM_DATASET_SIZE'])
    fof_name = os.getenv('CM_DATASET_SUBSET_FOF', 'files.txt')
    data_type = os.getenv('CM_DATASET_DATA_TYPE_INPUT', 'float32')
    data_layout = os.getenv('CM_DATASET_DATA_LAYOUT', '').lower()
    new_file_extension = os.getenv('CM_DATASET_PREPROCESSED_EXTENSION', '')
    normalize_data = int(os.getenv('CM_DATASET_NORMALIZE_DATA', '0'))
    subtract_mean = int(os.getenv('CM_DATASET_SUBTRACT_MEANS', '0'))
    given_channel_means = os.getenv('CM_DATASET_GIVEN_CHANNEL_MEANS', '')
    given_channel_stds = os.getenv('CM_DATASET_GIVEN_CHANNEL_STDS', '')
    quant_scale = float(os.environ['CM_DATASET_QUANT_SCALE'])
    quant_offset = float(os.environ['CM_DATASET_QUANT_OFFSET'])
    quantize = int(os.environ['CM_DATASET_QUANTIZE'])  # 1 for quantize to int8
    convert_to_unsigned = int(
        os.environ['CM_DATASET_CONVERT_TO_UNSIGNED'])  # 1 for int8 to uint8

    images_list = os.getenv('CM_DATASET_IMAGES_LIST')

    if given_channel_means:
        given_channel_means = [float(x)
                               for x in given_channel_means.split(' ')]

    if given_channel_stds:
        given_channel_stds = [float(x) for x in given_channel_stds.split(' ')]

    interpolation_method = os.getenv('CM_DATASET_INTERPOLATION_METHOD', '')

    print(("From: {}, To: {}, Size: {}, Crop: {}, InterSize: {}, 2BGR: {}, OFF: {}, VOL: '{}', FOF: {}," +
           " DTYPE: {}, DLAYOUT: {}, EXT: {}, NORM: {}, SMEAN: {}, GCM: {}, GSTD: {}, QUANTIZE: {}, QUANT_SCALE: {}, QUANT_OFFSET: {}, CONV_UNSIGNED: {}, INTER: {}").format(
        source_dir, destination_dir, square_side, crop_percentage, inter_size, convert_to_bgr, offset, volume, fof_name,
        data_type, data_layout, new_file_extension, normalize_data, subtract_mean, given_channel_means, given_channel_stds, quantize, quant_scale, quant_offset, convert_to_unsigned, interpolation_method))

    if interpolation_method == 'INTER_AREA':
        # Used for ResNet in pre_process_vgg.
        interpolation_method = cv2.INTER_AREA
    else:
        # Default interpolation method.
        interpolation_method = cv2.INTER_LINEAR

    filenames = []
    if images_list:
        with open(images_list) as f:
            filenames = f.read().splitlines()
    else:
        filenames = sorted(os.listdir(source_dir))

    if os.path.isdir(source_dir):
        sorted_filenames = [filename for filename in filenames if any(filename.lower().endswith(
            extension) for extension in supported_extensions) and not filename.startswith(".")]

        total_volume = len(sorted_filenames)

        if offset < 0:        # support offsets "from the right"
            offset += total_volume

        selected_filenames = sorted_filenames[offset:offset + volume]

    assert len(selected_filenames) == volume

    output_filenames = preprocess_files(
        selected_filenames, source_dir, destination_dir, crop_percentage, square_side, inter_size, convert_to_bgr,
        data_type, data_layout, new_file_extension, normalize_data, subtract_mean, given_channel_means, given_channel_stds, quantize, quant_scale, quant_offset, convert_to_unsigned, interpolation_method)

    fof_full_path = os.path.join(destination_dir, fof_name)
    with open(fof_full_path, 'w') as fof:
        for filename in output_filenames:
            fof.write(filename + '\n')
